@@ -32,9 +32,11 @@
 #include "command_executor.hh"
 
 #include "base/ansi_scrubber.hh"
+#include "base/ansi_vars.hh"
 #include "base/fs_util.hh"
 #include "base/injector.hh"
 #include "base/itertools.hh"
+#include "base/paths.hh"
 #include "base/string_util.hh"
 #include "bound_tags.hh"
 #include "config.h"
@@ -45,29 +47,25 @@
 #include "lnav_config.hh"
 #include "lnav_util.hh"
 #include "log_format_loader.hh"
+#include "prql-modules.h"
 #include "readline_highlighters.hh"
 #include "service_tags.hh"
 #include "shlex.hh"
+#include "sql_help.hh"
 #include "sql_util.hh"
 #include "vtab_module.hh"
-#include "yajlpp/json_ptr.hh"
+
+#ifdef HAVE_RUST_DEPS
+#    include "prqlc.cxx.hh"
+#endif
 
 using namespace lnav::roles::literals;
 
 exec_context INIT_EXEC_CONTEXT;
 
-static const std::string MSG_FORMAT_STMT = R"(
-SELECT count(*) AS total, min(log_line) AS log_line, log_msg_format
-    FROM all_logs
-    GROUP BY log_msg_format
-    ORDER BY total DESC
-)";
-
 int
 sql_progress(const struct log_cursor& lc)
 {
-    static sig_atomic_t sql_counter = 0;
-
     ssize_t total = lnav_data.ld_log_source.text_line_count();
     off_t off = lc.lc_curr_line;
 
@@ -83,42 +81,11 @@ sql_progress(const struct log_cursor& lc)
         return 1;
     }
 
+    static sig_atomic_t sql_counter = 0;
+
     if (ui_periodic_timer::singleton().time_to_update(sql_counter)) {
-        struct timeval current_time = {0, 0};
-        int ch;
-
-        while ((ch = getch()) != ERR) {
-            if (current_time.tv_sec == 0) {
-                gettimeofday(&current_time, nullptr);
-            }
-            lnav_data.ld_user_message_source.clear();
-
-            alerter::singleton().new_input(ch);
-
-            lnav_data.ld_input_dispatcher.new_input(current_time, ch);
-
-            lnav_data.ld_view_stack.top() | [ch](auto tc) {
-                lnav_data.ld_key_repeat_history.update(ch, tc->get_top());
-            };
-
-            if (!lnav_data.ld_looping) {
-                // No reason to keep processing input after the
-                // user has quit.  The view stack will also be
-                // empty, which will cause issues.
-                break;
-            }
-        }
-
         lnav_data.ld_bottom_source.update_loading(off, total);
-        lnav_data.ld_top_source.update_time();
-        lnav_data.ld_status[LNS_TOP].do_update();
-        lnav_data.ld_status[LNS_BOTTOM].do_update();
-        lnav_data.ld_rl_view->do_update();
-        if (handle_winch()) {
-            layout_views();
-            lnav_data.ld_view_stack.do_update();
-        }
-        refresh();
+        lnav_data.ld_status_refresher();
     }
 
     return 0;
@@ -132,15 +99,13 @@ sql_progress_finished()
     }
 
     lnav_data.ld_bottom_source.update_loading(0, 0);
-    lnav_data.ld_top_source.update_time();
-    lnav_data.ld_status[LNS_TOP].do_update();
-    lnav_data.ld_status[LNS_BOTTOM].do_update();
+    lnav_data.ld_status_refresher();
     lnav_data.ld_views[LNV_DB].redo_search();
 }
 
 static Result<std::string, lnav::console::user_message> execute_from_file(
     exec_context& ec,
-    const ghc::filesystem::path& path,
+    const std::string& src,
     int line_number,
     const std::string& cmdline);
 
@@ -201,14 +166,7 @@ bind_sql_parameters(exec_context& ec, sqlite3_stmt* stmt)
             return Err(um);
         }
 
-        ov_iter = ec.ec_override.find(name);
-        if (ov_iter != ec.ec_override.end()) {
-            sqlite3_bind_text(stmt,
-                              lpc,
-                              ov_iter->second.c_str(),
-                              ov_iter->second.length(),
-                              SQLITE_TRANSIENT);
-        } else if (name[0] == '$') {
+        if (name[0] == '$') {
             const auto& lvars = ec.ec_local_vars.top();
             const auto& gvars = ec.ec_global_vars;
             std::map<std::string, scoped_value_t>::const_iterator local_var,
@@ -298,14 +256,58 @@ execute_search(const std::string& search_cmd)
 Result<std::string, lnav::console::user_message>
 execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
 {
-    db_label_source& dls = lnav_data.ld_db_row_source;
+    db_label_source& dls = *(ec.ec_label_source_stack.back());
     auto_mem<sqlite3_stmt> stmt(sqlite3_finalize);
     struct timeval start_tv, end_tv;
     std::string stmt_str = trim(sql);
     std::string retval;
     int retcode = SQLITE_OK;
 
-    log_info("Executing SQL: %s", sql.c_str());
+    if (lnav::sql::is_prql(stmt_str)) {
+        log_info("compiling PRQL: %s", stmt_str.c_str());
+
+#if HAVE_RUST_DEPS
+        auto opts = prqlc::Options{true, "sql.sqlite", true};
+
+        auto tree = sqlite_extension_prql;
+        for (const auto& mod : lnav_prql_modules) {
+            tree.emplace_back(prqlc::SourceTreeElement{
+                mod.get_name(),
+                mod.to_string_fragment().to_string(),
+            });
+        }
+        tree.emplace_back(prqlc::SourceTreeElement{"", stmt_str});
+        auto cr = prqlc::compile_tree(tree, opts);
+
+        for (const auto& msg : cr.messages) {
+            if (msg.kind != prqlc::MessageKind::Error) {
+                continue;
+            }
+
+            auto stmt_al = attr_line_t(stmt_str);
+            readline_sqlite_highlighter(stmt_al, 0);
+            auto um
+                = lnav::console::user_message::error(
+                      attr_line_t("unable to compile PRQL: ").append(stmt_al))
+                      .with_reason((std::string) msg.reason);
+            if (!msg.display.empty()) {
+                um.with_note((std::string) msg.display);
+            }
+            for (const auto& hint : msg.hints) {
+                um.with_help(hint.data());
+                break;
+            }
+            return Err(um);
+        }
+        stmt_str = (std::string) cr.output;
+#else
+        auto um = lnav::console::user_message::error(
+            attr_line_t("PRQL is not supported in this build"));
+        return Err(um);
+#endif
+    }
+
+    log_info("Executing SQL: %s", stmt_str.c_str());
 
     auto old_mode = lnav_data.ld_mode;
     lnav_data.ld_mode = ln_mode_t::BUSY;
@@ -316,8 +318,9 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
         std::vector<std::string> args;
         split_ws(stmt_str, args);
 
-        auto* sql_cmd_map = injector::get<readline_context::command_map_t*,
-                                          sql_cmd_map_tag>();
+        const auto* sql_cmd_map
+            = injector::get<readline_context::command_map_t*,
+                            sql_cmd_map_tag>();
         auto cmd_iter = sql_cmd_map->find(args[0]);
 
         if (cmd_iter != sql_cmd_map->end()) {
@@ -329,10 +332,6 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
         }
     }
 
-    if (stmt_str == ".msgformats") {
-        stmt_str = MSG_FORMAT_STMT;
-    }
-
     ec.ec_accumulator->clear();
 
     const auto& source = ec.ec_source.back();
@@ -393,27 +392,19 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
         bool done = false;
 
         auto bound_values = TRY(bind_sql_parameters(ec, stmt.in()));
-        if (lnav_data.ld_rl_view != nullptr) {
-            if (lnav_data.ld_rl_view) {
-                lnav_data.ld_rl_view->set_attr_value(
-                    lnav::console::user_message::info(
-                        attr_line_t("executing SQL statement, press ")
-                            .append("CTRL+]"_hotkey)
-                            .append(" to cancel"))
-                        .to_attr_line());
-                lnav_data.ld_rl_view->do_update();
-            }
-        }
-
         ec.ec_sql_callback(ec, stmt.in());
         while (!done) {
             retcode = sqlite3_step(stmt.in());
 
             switch (retcode) {
                 case SQLITE_OK:
-                case SQLITE_DONE:
+                case SQLITE_DONE: {
+                    auto changes = sqlite3_changes(lnav_data.ld_db.in());
+
+                    log_info("sqlite3_changes() -> %d", changes);
                     done = true;
                     break;
+                }
 
                 case SQLITE_ROW:
                     ec.ec_sql_callback(ec, stmt.in());
@@ -449,7 +440,7 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
 
                     log_error("sqlite3_step error code: %d", retcode);
                     auto um = sqlite3_error_to_user_message(lnav_data.ld_db)
-                                  .with_snippets(ec.ec_source)
+                                  .with_context_snippets(ec.ec_source)
                                   .with_note(bound_note);
 
                     return Err(um);
@@ -536,6 +527,7 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
 
             if (lnav_data.ld_flags & LNF_HEADLESS) {
                 if (ec.ec_local_vars.size() == 1) {
+                    lnav_data.ld_views[LNV_DB].reload_data();
                     ensure_view(&lnav_data.ld_views[LNV_DB]);
                 }
             }
@@ -546,13 +538,73 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
     return Ok(retval);
 }
 
-static Result<std::string, lnav::console::user_message>
-execute_file_contents(exec_context& ec,
-                      const ghc::filesystem::path& path,
-                      bool multiline)
+Result<void, lnav::console::user_message>
+multiline_executor::push_back(string_fragment line)
 {
-    static ghc::filesystem::path stdin_path("-");
-    static ghc::filesystem::path dev_stdin_path("/dev/stdin");
+    this->me_line_number += 1;
+
+    if (line.trim().empty()) {
+        if (this->me_cmdline) {
+            this->me_cmdline = this->me_cmdline.value() + "\n";
+        }
+        return Ok();
+    }
+    if (line[0] == '#') {
+        return Ok();
+    }
+
+    switch (line[0]) {
+        case ':':
+        case '/':
+        case ';':
+        case '|':
+            if (this->me_cmdline) {
+                this->me_last_result
+                    = TRY(execute_from_file(this->me_exec_context,
+                                            this->me_source,
+                                            this->me_starting_line_number,
+                                            trim(this->me_cmdline.value())));
+            }
+
+            this->me_starting_line_number = this->me_line_number;
+            this->me_cmdline = line.to_string();
+            break;
+        default:
+            if (this->me_cmdline) {
+                this->me_cmdline = fmt::format(
+                    FMT_STRING("{}{}"), this->me_cmdline.value(), line);
+            } else {
+                this->me_last_result = TRY(
+                    execute_from_file(this->me_exec_context,
+                                      this->me_source,
+                                      this->me_line_number,
+                                      fmt::format(FMT_STRING(":{}"), line)));
+            }
+            break;
+    }
+
+    return Ok();
+}
+
+Result<std::string, lnav::console::user_message>
+multiline_executor::final()
+{
+    if (this->me_cmdline) {
+        this->me_last_result
+            = TRY(execute_from_file(this->me_exec_context,
+                                    this->me_source,
+                                    this->me_starting_line_number,
+                                    trim(this->me_cmdline.value())));
+    }
+
+    return Ok(this->me_last_result);
+}
+
+static Result<std::string, lnav::console::user_message>
+execute_file_contents(exec_context& ec, const ghc::filesystem::path& path)
+{
+    static const ghc::filesystem::path stdin_path("-");
+    static const ghc::filesystem::path dev_stdin_path("/dev/stdin");
 
     std::string retval;
     FILE* file;
@@ -562,59 +614,22 @@ execute_file_contents(exec_context& ec,
             return ec.make_error("stdin has already been consumed");
         }
         file = stdin;
-    } else if ((file = fopen(path.c_str(), "r")) == nullptr) {
+    } else if ((file = fopen(path.c_str(), "re")) == nullptr) {
         return ec.make_error("unable to open file");
     }
 
-    int line_number = 0, starting_line_number = 0;
     auto_mem<char> line;
     size_t line_max_size;
     ssize_t line_size;
-    nonstd::optional<std::string> cmdline;
+    multiline_executor me(ec, path.string());
 
     ec.ec_path_stack.emplace_back(path.parent_path());
     exec_context::output_guard og(ec);
     while ((line_size = getline(line.out(), &line_max_size, file)) != -1) {
-        line_number += 1;
-
-        if (trim(line.in()).empty()) {
-            if (multiline && cmdline) {
-                cmdline = cmdline.value() + "\n";
-            }
-            continue;
-        }
-        if (line[0] == '#') {
-            continue;
-        }
-
-        switch (line[0]) {
-            case ':':
-            case '/':
-            case ';':
-            case '|':
-                if (cmdline) {
-                    retval = TRY(execute_from_file(
-                        ec, path, starting_line_number, trim(cmdline.value())));
-                }
-
-                starting_line_number = line_number;
-                cmdline = std::string(line);
-                break;
-            default:
-                if (multiline) {
-                    cmdline = fmt::format("{}{}", cmdline.value(), line);
-                } else {
-                    retval = TRY(execute_from_file(
-                        ec, path, line_number, fmt::format(":{}", line.in())));
-                }
-                break;
-        }
+        TRY(me.push_back(string_fragment::from_bytes(line.in(), line_size)));
     }
 
-    if (cmdline) {
-        retval = TRY(execute_from_file(
-            ec, path, starting_line_number, trim(cmdline.value())));
-    }
+    retval = TRY(me.final());
 
     if (file == stdin) {
         if (isatty(STDOUT_FILENO)) {
@@ -629,25 +644,35 @@ execute_file_contents(exec_context& ec,
 }
 
 Result<std::string, lnav::console::user_message>
-execute_file(exec_context& ec, const std::string& path_and_args, bool multiline)
+execute_file(exec_context& ec, const std::string& path_and_args)
 {
+    static const intern_string_t SRC = intern_string::lookup("cmdline");
+
     available_scripts scripts;
-    std::vector<std::string> split_args;
     std::string retval, msg;
     shlex lexer(path_and_args);
 
     log_info("Executing file: %s", path_and_args.c_str());
 
-    if (!lexer.split(split_args, ec.ec_local_vars.top())) {
-        return ec.make_error("unable to parse path");
+    auto split_args_res = lexer.split(scoped_resolver{&ec.ec_local_vars.top()});
+    if (split_args_res.isErr()) {
+        auto split_err = split_args_res.unwrapErr();
+        auto um = lnav::console::user_message::error(
+                      "unable to parse script command-line")
+                      .with_reason(split_err.te_msg)
+                      .with_snippet(lnav::console::snippet::from(
+                          SRC, lexer.to_attr_line(split_err)));
+
+        return Err(um);
     }
+    auto split_args = split_args_res.unwrap();
     if (split_args.empty()) {
         return ec.make_error("no script specified");
     }
 
     ec.ec_local_vars.push({});
 
-    auto script_name = split_args[0];
+    auto script_name = split_args[0].se_value;
     auto& vars = ec.ec_local_vars.top();
     char env_arg_name[32];
     std::string star, open_error = "file not found";
@@ -660,13 +685,13 @@ execute_file(exec_context& ec, const std::string& path_and_args, bool multiline)
     vars["#"] = env_arg_name;
     for (size_t lpc = 0; lpc < split_args.size(); lpc++) {
         snprintf(env_arg_name, sizeof(env_arg_name), "%lu", lpc);
-        vars[env_arg_name] = split_args[lpc];
+        vars[env_arg_name] = split_args[lpc].se_value;
     }
     for (size_t lpc = 1; lpc < split_args.size(); lpc++) {
         if (lpc > 1) {
             star.append(" ");
         }
-        star.append(split_args[lpc]);
+        star.append(split_args[lpc].se_value);
     }
     vars["__all__"] = star;
 
@@ -707,8 +732,7 @@ execute_file(exec_context& ec, const std::string& path_and_args, bool multiline)
 
     if (!paths_to_exec.empty()) {
         for (auto& path_iter : paths_to_exec) {
-            retval
-                = TRY(execute_file_contents(ec, path_iter.sm_path, multiline));
+            retval = TRY(execute_file_contents(ec, path_iter.sm_path));
         }
     }
     ec.ec_local_vars.pop();
@@ -723,13 +747,13 @@ execute_file(exec_context& ec, const std::string& path_and_args, bool multiline)
 
 Result<std::string, lnav::console::user_message>
 execute_from_file(exec_context& ec,
-                  const ghc::filesystem::path& path,
+                  const std::string& src,
                   int line_number,
                   const std::string& cmdline)
 {
     std::string retval, alt_msg;
-    auto _sg = ec.enter_source(
-        intern_string::lookup(path.string()), line_number, cmdline);
+    auto _sg
+        = ec.enter_source(intern_string::lookup(src), line_number, cmdline);
 
     switch (cmdline[0]) {
         case ':':
@@ -750,10 +774,8 @@ execute_from_file(exec_context& ec,
             break;
     }
 
-    log_info("%s:%d:execute result -- %s",
-             path.c_str(),
-             line_number,
-             retval.c_str());
+    log_info(
+        "%s:%d:execute result -- %s", src.c_str(), line_number, retval.c_str());
 
     return Ok(retval);
 }
@@ -769,6 +791,7 @@ execute_any(exec_context& ec, const std::string& cmdline_with_mode)
             (lnav_data.ld_flags & LNF_HEADLESS || ec.ec_path_stack.size() > 1))
         {
             rescan_files();
+            wait_for_pipers(nonstd::nullopt);
             rebuild_indexes_repeatedly();
         }
     });
@@ -820,11 +843,13 @@ execute_init_commands(
                               "");
             return;
         }
+        fcntl(fileno(tmpout), F_SETFD, FD_CLOEXEC);
         fd_copy = auto_fd::dup_of(fileno(tmpout));
+        fd_copy.close_on_exec();
         ec_out = std::make_pair(tmpout.release(), fclose);
     }
 
-    auto& dls = lnav_data.ld_db_row_source;
+    auto& dls = *(ec.ec_label_source_stack.back());
     int option_index = 1;
 
     {
@@ -839,6 +864,8 @@ execute_init_commands(
 
             wait_for_children();
 
+            lnav_data.ld_view_stack.top() |
+                [&ec](auto* tc) { ec.ec_top_line = tc->get_selection(); };
             log_debug("init cmd: %s", cmd.c_str());
             {
                 auto _sg
@@ -863,9 +890,16 @@ execute_init_commands(
                 }
 
                 rescan_files();
+                auto deadline = current_timeval()
+                    + ((lnav_data.ld_flags & LNF_HEADLESS)
+                           ? timeval{5, 0}
+                           : timeval{0, 500000});
+                wait_for_pipers(deadline);
                 rebuild_indexes_repeatedly();
             }
-            if (dls.dls_rows.size() > 1) {
+            if (dls.dls_rows.size() > 1 && lnav_data.ld_view_stack.size() == 1)
+            {
+                lnav_data.ld_views[LNV_DB].reload_data();
                 ensure_view(LNV_DB);
             }
         }
@@ -877,15 +911,20 @@ execute_init_commands(
     if (ec_out && fstat(fd_copy, &st) != -1 && st.st_size > 0) {
         static const auto OUTPUT_NAME = std::string("Initial command output");
 
-        lnav_data.ld_active_files.fc_file_names[OUTPUT_NAME]
-            .with_fd(std::move(fd_copy))
-            .with_include_in_session(false)
-            .with_detect_format(false);
-        lnav_data.ld_files_to_front.emplace_back(OUTPUT_NAME, 0_vl);
+        auto create_piper_res = lnav::piper::create_looper(
+            OUTPUT_NAME, std::move(fd_copy), auto_fd{});
+        if (create_piper_res.isOk()) {
+            lnav_data.ld_active_files.fc_file_names[OUTPUT_NAME]
+                .with_piper(create_piper_res.unwrap())
+                .with_include_in_session(false)
+                .with_detect_format(false)
+                .with_init_location(0_vl);
+            lnav_data.ld_files_to_front.emplace_back(OUTPUT_NAME, 0_vl);
 
-        if (lnav_data.ld_rl_view != nullptr) {
-            lnav_data.ld_rl_view->set_alt_value(
-                HELP_MSG_1(X, "to close the file"));
+            if (lnav_data.ld_rl_view != nullptr) {
+                lnav_data.ld_rl_view->set_alt_value(
+                    HELP_MSG_1(X, "to close the file"));
+            }
         }
     }
 
@@ -895,7 +934,7 @@ execute_init_commands(
 int
 sql_callback(exec_context& ec, sqlite3_stmt* stmt)
 {
-    auto& dls = lnav_data.ld_db_row_source;
+    auto& dls = *(ec.ec_label_source_stack.back());
 
     if (!sqlite3_stmt_busy(stmt)) {
         dls.clear();
@@ -925,8 +964,17 @@ sql_callback(exec_context& ec, sqlite3_stmt* stmt)
 
             dls.push_header(colname, type, graphable);
             if (graphable) {
-                auto attrs = vc.attrs_for_ident(colname);
+                auto name_for_ident_attrs = colname;
+                auto attrs = vc.attrs_for_ident(name_for_ident_attrs);
+                for (size_t attempt = 0;
+                     chart.attrs_in_use(attrs) && attempt < 3;
+                     attempt++)
+                {
+                    name_for_ident_attrs += " ";
+                    attrs = vc.attrs_for_ident(name_for_ident_attrs);
+                }
                 chart.with_attrs_for_ident(colname, attrs);
+                dls.dls_headers.back().hm_title_attrs = attrs;
             }
         }
         set_vars = true;
@@ -989,7 +1037,7 @@ pipe_callback(exec_context& ec, const std::string& cmdline, auto_fd& fd)
     if (out) {
         FILE* file = *out;
 
-        return std::async(std::launch::async, [&fd, file]() {
+        return std::async(std::launch::async, [fd = std::move(fd), file]() {
             char buffer[1024];
             ssize_t rc;
 
@@ -1004,26 +1052,35 @@ pipe_callback(exec_context& ec, const std::string& cmdline, auto_fd& fd)
             return std::string();
         });
     }
-    auto tmp_fd
-        = lnav::filesystem::open_temp_file(
-              ghc::filesystem::temp_directory_path() / "lnav.out.XXXXXX")
-              .map([](auto pair) {
-                  ghc::filesystem::remove(pair.first);
+    auto open_temp_res = lnav::filesystem::open_temp_file(lnav::paths::workdir()
+                                                          / "exec.XXXXXX");
+    if (open_temp_res.isErr()) {
+        return lnav::futures::make_ready_future(
+            fmt::format(FMT_STRING("error: cannot open temp file -- {}"),
+                        open_temp_res.unwrapErr()));
+    }
+
+    auto tmp_pair = open_temp_res.unwrap();
+
+    auto reader = std::thread(
+        [in_fd = std::move(fd), out_fd = std::move(tmp_pair.second)]() {
+            char buffer[1024];
+            ssize_t rc;
+
+            while ((rc = read(in_fd, buffer, sizeof(buffer))) > 0) {
+                write(out_fd, buffer, rc);
+            }
+        });
+    reader.detach();
 
-                  return std::move(pair.second);
-              })
-              .expect("Cannot create temporary file for callback");
-    auto pp
-        = std::make_shared<piper_proc>(std::move(fd), false, std::move(tmp_fd));
     static int exec_count = 0;
-
-    lnav_data.ld_pipers.push_back(pp);
     auto desc
         = fmt::format(FMT_STRING("[{}] Output of {}"), exec_count++, cmdline);
-    lnav_data.ld_active_files.fc_file_names[desc]
-        .with_fd(pp->get_fd())
+    lnav_data.ld_active_files.fc_file_names[tmp_pair.first]
+        .with_filename(desc)
         .with_include_in_session(false)
-        .with_detect_format(false);
+        .with_detect_format(false)
+        .with_init_location(0_vl);
     lnav_data.ld_files_to_front.emplace_back(desc, 0_vl);
     if (lnav_data.ld_rl_view != nullptr) {
         lnav_data.ld_rl_view->set_alt_value(HELP_MSG_1(X, "to close the file"));
@@ -1039,7 +1096,7 @@ add_global_vars(exec_context& ec)
         shlex subber(iter.second);
         std::string str;
 
-        if (!subber.eval(str, ec.ec_global_vars)) {
+        if (!subber.eval(str, scoped_resolver{&ec.ec_global_vars})) {
             log_error("Unable to evaluate global variable value: %s",
                       iter.second.c_str());
             continue;
