@@ -102,6 +102,11 @@ hist_source2::text_attrs_for_line(textview_curses& tc,
                                              bucket.b_values[lpc].hv_value,
                                              value_out);
     }
+    auto alt_row_index = row % 4;
+    if (alt_row_index == 2 || alt_row_index == 3) {
+        value_out.emplace_back(line_range{0, -1},
+                               VC_ROLE.value(role_t::VCR_ALT_ROW));
+    }
 }
 
 void
@@ -109,11 +114,7 @@ hist_source2::add_value(time_t row,
                         hist_source2::hist_type_t htype,
                         double value)
 {
-    if (row < this->hs_last_row) {
-        log_error("time mismatch %ld %ld", row, this->hs_last_row);
-    }
-
-    require(row >= this->hs_last_row);
+    require_ge(row, this->hs_last_row);
 
     row = rounddown(row, this->hs_time_slice);
     if (row != this->hs_last_row) {
@@ -123,7 +124,7 @@ hist_source2::add_value(time_t row,
         this->hs_last_row = row;
     }
 
-    bucket_t& bucket = this->find_bucket(this->hs_last_bucket);
+    auto& bucket = this->find_bucket(this->hs_last_bucket);
     bucket.b_time = row;
     bucket.b_values[htype].hv_value += value;
 }
@@ -133,7 +134,7 @@ hist_source2::init()
 {
     view_colors& vc = view_colors::singleton();
 
-    this->hs_chart
+    this->hs_chart.with_show_state(stacked_bar_chart_base::show_all{})
         .with_attrs_for_ident(HT_NORMAL, vc.attrs_for_role(role_t::VCR_TEXT))
         .with_attrs_for_ident(HT_WARNING,
                               vc.attrs_for_role(role_t::VCR_WARNING))
@@ -165,7 +166,7 @@ hist_source2::end_of_row()
     }
 }
 
-nonstd::optional<struct timeval>
+nonstd::optional<text_time_translator::row_info>
 hist_source2::time_for_row(vis_line_t row)
 {
     if (row < 0 || row > this->hs_line_count) {
@@ -174,7 +175,7 @@ hist_source2::time_for_row(vis_line_t row)
 
     bucket_t& bucket = this->find_bucket(row);
 
-    return timeval{bucket.b_time, 0};
+    return row_info{timeval{bucket.b_time, 0}, row};
 }
 
 hist_source2::bucket_t&
