@@ -81,21 +81,36 @@ lnav_config_listener* lnav_config_listener::LISTENER_LIST;
 static auto a = injector::bind<archive_manager::config>::to_instance(
     +[]() { return &lnav_config.lc_archive_manager; });
 
+static auto dtc = injector::bind<date_time_scanner_ns::config>::to_instance(
+    +[]() { return &lnav_config.lc_log_date_time; });
+
 static auto fvc = injector::bind<file_vtab::config>::to_instance(
     +[]() { return &lnav_config.lc_file_vtab; });
 
 static auto lc = injector::bind<lnav::logfile::config>::to_instance(
     +[]() { return &lnav_config.lc_logfile; });
 
+static auto p = injector::bind<lnav::piper::config>::to_instance(
+    +[]() { return &lnav_config.lc_piper; });
+
 static auto tc = injector::bind<tailer::config>::to_instance(
     +[]() { return &lnav_config.lc_tailer; });
 
 static auto scc = injector::bind<sysclip::config>::to_instance(
     +[]() { return &lnav_config.lc_sysclip; });
 
+static auto uh = injector::bind<lnav::url_handler::config>::to_instance(
+    +[]() { return &lnav_config.lc_url_handlers; });
+
 static auto lsc = injector::bind<logfile_sub_source_ns::config>::to_instance(
     +[]() { return &lnav_config.lc_log_source; });
 
+static auto annoc = injector::bind<lnav::log::annotate::config>::to_instance(
+    +[]() { return &lnav_config.lc_log_annotations; });
+
+static auto tssc = injector::bind<top_status_source_cfg>::to_instance(
+    +[]() { return &lnav_config.lc_top_status_cfg; });
+
 bool
 check_experimental(const char* feature_name)
 {
@@ -131,10 +146,30 @@ ensure_dotlnav()
     for (const auto* sub_path : subdirs) {
         auto full_path = path / sub_path;
 
-        log_perror(mkdir(full_path.c_str(), 0755));
+        if (mkdir(full_path.c_str(), 0755) == -1 && errno != EEXIST) {
+            log_error("unable to make directory: %s -- %s",
+                      full_path.c_str(),
+                      strerror(errno));
+        }
     }
 
-    lnav_log_crash_dir = strdup(path.c_str());
+    auto crash_dir_path = path / "crash";
+    lnav_log_crash_dir = strdup(crash_dir_path.c_str());
+
+    {
+        static_root_mem<glob_t, globfree> gl;
+        auto crash_glob = path / "crash-*";
+
+        if (glob(crash_glob.c_str(), GLOB_NOCHECK, nullptr, gl.inout()) == 0) {
+            std::error_code ec;
+            for (size_t lpc = 0; lpc < gl->gl_pathc; lpc++) {
+                auto crash_file = ghc::filesystem::path(gl->gl_pathv[lpc]);
+
+                ghc::filesystem::rename(
+                    crash_file, crash_dir_path / crash_file.filename(), ec);
+            }
+        }
+    }
 
     {
         static_root_mem<glob_t, globfree> gl;
@@ -221,19 +256,23 @@ install_from_git(const std::string& repo)
     }
 
     auto finished_child = std::move(git_cmd).wait_for_child();
-
     if (!finished_child.was_normal_exit() || finished_child.exit_status() != 0)
     {
         return false;
     }
 
+    if (ghc::filesystem::is_directory(local_formats_path)
+        || ghc::filesystem::is_directory(local_configs_path))
+    {
+        return false;
+    }
     if (!ghc::filesystem::is_directory(local_staging_path)) {
         auto um
             = lnav::console::user_message::error(
                   attr_line_t("failed to install git repo: ")
                       .append(lnav::roles::file(repo)))
                   .with_reason(
-                      attr_line_t("git failed to create the local directory")
+                      attr_line_t("git failed to create the local directory ")
                           .append(
                               lnav::roles::file(local_staging_path.string())));
         lnav::console::print(stderr, um);
@@ -333,13 +372,14 @@ update_installs_from_git()
     auto git_formats = lnav::paths::dotlnav() / "formats/*/.git";
     bool found = false, retval = true;
 
-    if (glob(git_formats.c_str(), GLOB_NOCHECK, nullptr, gl.inout()) == 0) {
+    if (glob(git_formats.c_str(), 0, nullptr, gl.inout()) == 0) {
         for (int lpc = 0; lpc < (int) gl->gl_pathc; lpc++) {
-            char* git_dir = dirname(gl->gl_pathv[lpc]);
+            auto git_dir
+                = ghc::filesystem::path(gl->gl_pathv[lpc]).parent_path();
 
-            printf("Updating formats in %s\n", git_dir);
-            auto pull_cmd
-                = fmt::format(FMT_STRING("cd '{}' && git pull"), git_dir);
+            printf("Updating formats in %s\n", git_dir.c_str());
+            auto pull_cmd = fmt::format(FMT_STRING("cd '{}' && git pull"),
+                                        git_dir.string());
             int ret = system(pull_cmd.c_str());
             if (ret == -1) {
                 std::cerr << "Failed to spawn command "
@@ -359,7 +399,7 @@ update_installs_from_git()
     if (!found) {
         printf(
             "No formats from git repositories found, "
-            "use 'lnav -i extra' to install third-party foramts\n");
+            "use 'lnav -i extra' to install third-party formats\n");
     }
 
     return retval;
@@ -429,9 +469,11 @@ install_extra_formats()
     }
 }
 
-struct userdata {
-    explicit userdata(std::vector<lnav::console::user_message>& errors)
-        : ud_errors(errors){};
+struct config_userdata {
+    explicit config_userdata(std::vector<lnav::console::user_message>& errors)
+        : ud_errors(errors)
+    {
+    }
 
     std::vector<lnav::console::user_message>& ud_errors;
 };
@@ -440,7 +482,7 @@ static void
 config_error_reporter(const yajlpp_parse_context& ypc,
                       const lnav::console::user_message& msg)
 {
-    auto* ud = (userdata*) ypc.ypc_userdata;
+    auto* ud = (config_userdata*) ypc.ypc_userdata;
 
     ud->ud_errors.emplace_back(msg);
 }
@@ -471,9 +513,7 @@ static const struct json_path_container keymap_def_handlers = {
             "an 'x' followed by the hexadecimal representation of the byte.")
         .with_obj_provider<key_command, key_map>(
             [](const yajlpp_provider_context& ypc, key_map* km) {
-                key_command& retval
-                    = km->km_seq_to_cmd[ypc.ypc_extractor.get_substr(
-                        "key_seq")];
+                auto& retval = km->km_seq_to_cmd[ypc.get_substr("key_seq")];
 
                 return &retval;
             })
@@ -492,8 +532,7 @@ static const struct json_path_container keymap_defs_handlers = {
         .with_obj_provider<key_map, _lnav_config>(
             [](const yajlpp_provider_context& ypc, _lnav_config* root) {
                 key_map& retval
-                    = root->lc_ui_keymaps[ypc.ypc_extractor.get_substr(
-                        "keymap_name")];
+                    = root->lc_ui_keymaps[ypc.get_substr("keymap_name")];
                 return &retval;
             })
         .with_path_provider<_lnav_config>(
@@ -505,6 +544,23 @@ static const struct json_path_container keymap_defs_handlers = {
         .with_children(keymap_def_handlers),
 };
 
+static const json_path_handler_base::enum_value_t _movement_values[] = {
+    {"top", config_movement_mode::TOP},
+    {"cursor", config_movement_mode::CURSOR},
+
+    json_path_handler_base::ENUM_TERMINATOR,
+};
+
+static const struct json_path_container movement_handlers = {
+    yajlpp::property_handler("mode")
+        .with_synopsis("top|cursor")
+        .with_enum_values(_movement_values)
+        .with_example("top")
+        .with_example("cursor")
+        .with_description("The mode of cursor movement to use.")
+        .for_field<>(&_lnav_config::lc_ui_movement, &movement_config::mode),
+};
+
 static const struct json_path_container global_var_handlers = {
     yajlpp::pattern_property_handler("(?<var_name>\\w+)")
         .with_synopsis("<name>")
@@ -517,7 +573,7 @@ static const struct json_path_container global_var_handlers = {
                     paths_out.emplace_back(iter.first);
                 }
             })
-        .FOR_FIELD(_lnav_config, lc_global_vars),
+        .for_field(&_lnav_config::lc_global_vars),
 };
 
 static const struct json_path_container style_config_handlers =
@@ -583,6 +639,14 @@ static const struct json_path_container theme_styles_handlers = {
         .with_description("Styling for hidden fields in logs")
         .for_child(&lnav_theme::lt_style_hidden)
         .with_children(style_config_handlers),
+    yajlpp::property_handler("cursor-line")
+        .with_description("Styling for the cursor line in the main view")
+        .for_child(&lnav_theme::lt_style_cursor_line)
+        .with_children(style_config_handlers),
+    yajlpp::property_handler("disabled-cursor-line")
+        .with_description("Styling for the cursor line when it is disabled")
+        .for_child(&lnav_theme::lt_style_disabled_cursor_line)
+        .with_children(style_config_handlers),
     yajlpp::property_handler("adjusted-time")
         .with_description("Styling for timestamps that have been adjusted")
         .for_child(&lnav_theme::lt_style_adjusted_time)
@@ -592,8 +656,12 @@ static const struct json_path_container theme_styles_handlers = {
             "Styling for timestamps that are different from the received time")
         .for_child(&lnav_theme::lt_style_skewed_time)
         .with_children(style_config_handlers),
+    yajlpp::property_handler("file-offset")
+        .with_description("Styling for a file offset")
+        .for_child(&lnav_theme::lt_style_file_offset)
+        .with_children(style_config_handlers),
     yajlpp::property_handler("offset-time")
-        .with_description("Styling for hidden fields")
+        .with_description("Styling for the elapsed time column")
         .for_child(&lnav_theme::lt_style_offset_time)
         .with_children(style_config_handlers),
     yajlpp::property_handler("invalid-msg")
@@ -620,6 +688,12 @@ static const struct json_path_container theme_styles_handlers = {
         .with_description("Styling for top-level headers")
         .with_obj_provider<style_config, lnav_theme>(
             [](const yajlpp_provider_context& ypc, lnav_theme* root) {
+                if (ypc.ypc_parse_context != nullptr
+                    && root->lt_style_header[0].pp_path.empty())
+                {
+                    root->lt_style_header[0].pp_path
+                        = ypc.ypc_parse_context->get_full_path();
+                }
                 return &root->lt_style_header[0].pp_value;
             })
         .with_children(style_config_handlers),
@@ -627,6 +701,12 @@ static const struct json_path_container theme_styles_handlers = {
         .with_description("Styling for 2nd-level headers")
         .with_obj_provider<style_config, lnav_theme>(
             [](const yajlpp_provider_context& ypc, lnav_theme* root) {
+                if (ypc.ypc_parse_context != nullptr
+                    && root->lt_style_header[1].pp_path.empty())
+                {
+                    root->lt_style_header[1].pp_path
+                        = ypc.ypc_parse_context->get_full_path();
+                }
                 return &root->lt_style_header[1].pp_value;
             })
         .with_children(style_config_handlers),
@@ -634,6 +714,12 @@ static const struct json_path_container theme_styles_handlers = {
         .with_description("Styling for 3rd-level headers")
         .with_obj_provider<style_config, lnav_theme>(
             [](const yajlpp_provider_context& ypc, lnav_theme* root) {
+                if (ypc.ypc_parse_context != nullptr
+                    && root->lt_style_header[2].pp_path.empty())
+                {
+                    root->lt_style_header[2].pp_path
+                        = ypc.ypc_parse_context->get_full_path();
+                }
                 return &root->lt_style_header[2].pp_value;
             })
         .with_children(style_config_handlers),
@@ -641,6 +727,12 @@ static const struct json_path_container theme_styles_handlers = {
         .with_description("Styling for 4th-level headers")
         .with_obj_provider<style_config, lnav_theme>(
             [](const yajlpp_provider_context& ypc, lnav_theme* root) {
+                if (ypc.ypc_parse_context != nullptr
+                    && root->lt_style_header[3].pp_path.empty())
+                {
+                    root->lt_style_header[3].pp_path
+                        = ypc.ypc_parse_context->get_full_path();
+                }
                 return &root->lt_style_header[3].pp_value;
             })
         .with_children(style_config_handlers),
@@ -648,6 +740,12 @@ static const struct json_path_container theme_styles_handlers = {
         .with_description("Styling for 5th-level headers")
         .with_obj_provider<style_config, lnav_theme>(
             [](const yajlpp_provider_context& ypc, lnav_theme* root) {
+                if (ypc.ypc_parse_context != nullptr
+                    && root->lt_style_header[4].pp_path.empty())
+                {
+                    root->lt_style_header[4].pp_path
+                        = ypc.ypc_parse_context->get_full_path();
+                }
                 return &root->lt_style_header[4].pp_value;
             })
         .with_children(style_config_handlers),
@@ -655,6 +753,12 @@ static const struct json_path_container theme_styles_handlers = {
         .with_description("Styling for 6th-level headers")
         .with_obj_provider<style_config, lnav_theme>(
             [](const yajlpp_provider_context& ypc, lnav_theme* root) {
+                if (ypc.ypc_parse_context != nullptr
+                    && root->lt_style_header[5].pp_path.empty())
+                {
+                    root->lt_style_header[5].pp_path
+                        = ypc.ypc_parse_context->get_full_path();
+                }
                 return &root->lt_style_header[5].pp_value;
             })
         .with_children(style_config_handlers),
@@ -702,9 +806,17 @@ static const struct json_path_container theme_styles_handlers = {
         .with_description("Styling for snippet borders")
         .for_child(&lnav_theme::lt_style_snippet_border)
         .with_children(style_config_handlers),
+    yajlpp::property_handler("indent-guide")
+        .with_description("Styling for indent guide lines")
+        .for_child(&lnav_theme::lt_style_indent_guide)
+        .with_children(style_config_handlers),
 };
 
 static const struct json_path_container theme_syntax_styles_handlers = {
+    yajlpp::property_handler("inline-code")
+        .with_description("Styling for inline code blocks")
+        .for_child(&lnav_theme::lt_style_inline_code)
+        .with_children(style_config_handlers),
     yajlpp::property_handler("quoted-code")
         .with_description("Styling for quoted code blocks")
         .for_child(&lnav_theme::lt_style_quoted_code)
@@ -738,10 +850,35 @@ static const struct json_path_container theme_syntax_styles_handlers = {
         .with_description("Styling for symbols in source files")
         .for_child(&lnav_theme::lt_style_symbol)
         .with_children(style_config_handlers),
+    yajlpp::property_handler("null")
+        .with_description("Styling for nulls in source files")
+        .for_child(&lnav_theme::lt_style_null)
+        .with_children(style_config_handlers),
+    yajlpp::property_handler("ascii-control")
+        .with_description(
+            "Styling for ASCII control characters in source files")
+        .for_child(&lnav_theme::lt_style_ascii_ctrl)
+        .with_children(style_config_handlers),
+    yajlpp::property_handler("non-ascii")
+        .with_description("Styling for non-ASCII characters in source files")
+        .for_child(&lnav_theme::lt_style_non_ascii)
+        .with_children(style_config_handlers),
     yajlpp::property_handler("number")
         .with_description("Styling for numbers in source files")
         .for_child(&lnav_theme::lt_style_number)
         .with_children(style_config_handlers),
+    yajlpp::property_handler("type")
+        .with_description("Styling for types in source files")
+        .for_child(&lnav_theme::lt_style_type)
+        .with_children(style_config_handlers),
+    yajlpp::property_handler("function")
+        .with_description("Styling for functions in source files")
+        .for_child(&lnav_theme::lt_style_function)
+        .with_children(style_config_handlers),
+    yajlpp::property_handler("separators-references-accessors")
+        .with_description("Styling for sigils in source files")
+        .for_child(&lnav_theme::lt_style_sep_ref_acc)
+        .with_children(style_config_handlers),
     yajlpp::property_handler("re-special")
         .with_description(
             "Styling for special characters in regular expressions")
@@ -836,6 +973,10 @@ static const struct json_path_container theme_status_styles_handlers = {
         .with_description("Styling for hotkey highlights of status bars")
         .for_child(&lnav_theme::lt_style_status_hotkey)
         .with_children(style_config_handlers),
+    yajlpp::property_handler("suggestion")
+        .with_description("Styling for suggested values")
+        .for_child(&lnav_theme::lt_style_suggestion)
+        .with_children(style_config_handlers),
 };
 
 static const struct json_path_container theme_log_level_styles_handlers = {
@@ -845,7 +986,7 @@ static const struct json_path_container theme_log_level_styles_handlers = {
         .with_obj_provider<style_config, lnav_theme>(
             [](const yajlpp_provider_context& ypc, lnav_theme* root) {
                 auto& sc = root->lt_level_styles[string2level(
-                    ypc.ypc_extractor.get_substr_i("level").get())];
+                    ypc.get_substr_i("level").get())];
 
                 if (ypc.ypc_parse_context != nullptr && sc.pp_path.empty()) {
                     sc.pp_path = ypc.ypc_parse_context->get_full_path();
@@ -866,7 +1007,7 @@ static const struct json_path_container highlighter_handlers = {
     yajlpp::property_handler("pattern")
         .with_synopsis("regular expression")
         .with_description("The regular expression to highlight")
-        .FOR_FIELD(highlighter_config, hc_regex),
+        .for_field(&highlighter_config::hc_regex),
 
     yajlpp::property_handler("style")
         .with_description(
@@ -877,15 +1018,14 @@ static const struct json_path_container highlighter_handlers = {
 
 static const struct json_path_container theme_highlights_handlers = {
     yajlpp::pattern_property_handler("(?<highlight_name>[\\w\\-]+)")
-        .with_obj_provider<highlighter_config, lnav_theme>(
-            [](const yajlpp_provider_context& ypc, lnav_theme* root) {
-                highlighter_config& hc
-                    = root->lt_highlights[ypc.ypc_extractor
-                                              .get_substr_i("highlight_name")
-                                              .get()];
+        .with_obj_provider<highlighter_config,
+                           lnav_theme>([](const yajlpp_provider_context& ypc,
+                                          lnav_theme* root) {
+            highlighter_config& hc
+                = root->lt_highlights[ypc.get_substr_i("highlight_name").get()];
 
-                return &hc;
-            })
+            return &hc;
+        })
         .with_path_provider<lnav_theme>(
             [](struct lnav_theme* cfg, std::vector<std::string>& paths_out) {
                 for (const auto& pair : cfg->lt_highlights) {
@@ -905,7 +1045,7 @@ static const struct json_path_container theme_vars_handlers = {
                     paths_out.emplace_back(iter.first);
                 }
             })
-        .FOR_FIELD(lnav_theme, lt_vars),
+        .for_field(&lnav_theme::lt_vars),
 };
 
 static const struct json_path_container theme_def_handlers = {
@@ -940,8 +1080,7 @@ static const struct json_path_container theme_defs_handlers = {
         .with_obj_provider<lnav_theme, _lnav_config>(
             [](const yajlpp_provider_context& ypc, _lnav_config* root) {
                 lnav_theme& lt
-                    = root->lc_ui_theme_defs[ypc.ypc_extractor.get_substr(
-                        "theme_name")];
+                    = root->lc_ui_theme_defs[ypc.get_substr("theme_name")];
 
                 return &lt;
             })
@@ -953,8 +1092,7 @@ static const struct json_path_container theme_defs_handlers = {
             })
         .with_obj_deleter(
             +[](const yajlpp_provider_context& ypc, _lnav_config* root) {
-                root->lc_ui_theme_defs.erase(
-                    ypc.ypc_extractor.get_substr("theme_name"));
+                root->lc_ui_theme_defs.erase(ypc.get_substr("theme_name"));
             })
         .with_children(theme_def_handlers),
 };
@@ -965,7 +1103,8 @@ static const struct json_path_container ui_handlers = {
         .with_description("The format for the clock displayed in "
                           "the top-left corner using strftime(3) conversions")
         .with_example("%a %b %d %H:%M:%S %Z")
-        .for_field(&_lnav_config::lc_ui_clock_format),
+        .for_field(&_lnav_config::lc_top_status_cfg,
+                   &top_status_source_cfg::tssc_clock_format),
     yajlpp::property_handler("dim-text")
         .with_synopsis("bool")
         .with_description("Reduce the brightness of text (useful for xterms). "
@@ -991,6 +1130,9 @@ static const struct json_path_container ui_handlers = {
     yajlpp::property_handler("theme-defs")
         .with_description("Theme definitions.")
         .with_children(theme_defs_handlers),
+    yajlpp::property_handler("movement")
+        .with_description("Log file cursor movement mode settings")
+        .with_children(movement_handlers),
     yajlpp::property_handler("keymap-defs")
         .with_description("Keymap definitions.")
         .with_children(keymap_defs_handlers),
@@ -1016,6 +1158,27 @@ static const struct json_path_container archive_handlers = {
                    &archive_manager::config::amc_cache_ttl),
 };
 
+static const struct json_path_container piper_handlers = {
+    yajlpp::property_handler("max-size")
+        .with_synopsis("<bytes>")
+        .with_description("The maximum size of a capture file")
+        .with_min_value(128)
+        .for_field(&_lnav_config::lc_piper, &lnav::piper::config::c_max_size),
+    yajlpp::property_handler("rotations")
+        .with_synopsis("<count>")
+        .with_min_value(2)
+        .with_description("The number of rotated files to keep")
+        .for_field(&_lnav_config::lc_piper, &lnav::piper::config::c_rotations),
+    yajlpp::property_handler("ttl")
+        .with_synopsis("<duration>")
+        .with_description(
+            "The time-to-live for captured data, expressed as a duration "
+            "(e.g. '3d' for three days)")
+        .with_example("3d")
+        .with_example("12h")
+        .for_field(&_lnav_config::lc_piper, &lnav::piper::config::c_ttl),
+};
+
 static const struct json_path_container file_vtab_handlers = {
     yajlpp::property_handler("max-content-size")
         .with_synopsis("<bytes>")
@@ -1137,9 +1300,8 @@ static const struct json_path_container sysclip_impls_handlers = {
         .with_obj_provider<sysclip::clipboard, _lnav_config>(
             [](const yajlpp_provider_context& ypc, _lnav_config* root) {
                 auto& retval
-                    = root->lc_sysclip
-                          .c_clipboard_impls[ypc.ypc_extractor.get_substr(
-                              "clipboard_impl_name")];
+                    = root->lc_sysclip.c_clipboard_impls[ypc.get_substr(
+                        "clipboard_impl_name")];
                 return &retval;
             })
         .with_path_provider<_lnav_config>(
@@ -1171,15 +1333,14 @@ static const struct json_path_container log_source_watch_expr_handlers = {
 };
 
 static const struct json_path_container log_source_watch_handlers = {
-    yajlpp::pattern_property_handler("(?<watch_name>[\\w\\-]+)")
+    yajlpp::pattern_property_handler("(?<watch_name>[\\w\\.\\-]+)")
         .with_synopsis("<name>")
         .with_description("A log message watch expression")
         .with_obj_provider<logfile_sub_source_ns::watch_expression,
                            _lnav_config>(
             [](const yajlpp_provider_context& ypc, _lnav_config* root) {
                 auto& retval = root->lc_log_source
-                                   .c_watch_exprs[ypc.ypc_extractor.get_substr(
-                                       "watch_name")];
+                                   .c_watch_exprs[ypc.get_substr("watch_name")];
                 return &retval;
             })
         .with_path_provider<_lnav_config>(
@@ -1191,21 +1352,103 @@ static const struct json_path_container log_source_watch_handlers = {
         .with_obj_deleter(
             +[](const yajlpp_provider_context& ypc, _lnav_config* root) {
                 root->lc_log_source.c_watch_exprs.erase(
-                    ypc.ypc_extractor.get_substr("watch_name"));
+                    ypc.get_substr("watch_name"));
             })
         .with_children(log_source_watch_expr_handlers),
 };
 
+static const struct json_path_container annotation_handlers = {
+    yajlpp::property_handler("description")
+        .with_synopsis("<text>")
+        .with_description("A description of this annotation")
+        .for_field(&lnav::log::annotate::annotation_def::a_description),
+    yajlpp::property_handler("condition")
+        .with_synopsis("<SQL-expression>")
+        .with_description(
+            "The SQLite expression to execute for a log message that "
+            "determines whether or not this annotation is applicable.  The "
+            "expression is evaluated the same way as a filter expression")
+        .with_min_length(1)
+        .for_field(&lnav::log::annotate::annotation_def::a_condition),
+    yajlpp::property_handler("handler")
+        .with_synopsis("<script>")
+        .with_description("The script to execute to generate the annotation "
+                          "content. A JSON object with the log message content "
+                          "will be sent to the script on the standard input")
+        .with_min_length(1)
+        .for_field(&lnav::log::annotate::annotation_def::a_handler),
+};
+
+static const struct json_path_container annotations_handlers = {
+    yajlpp::pattern_property_handler(R"((?<annotation_name>[\w\.\-]+))")
+        .with_obj_provider<lnav::log::annotate::annotation_def, _lnav_config>(
+            [](const yajlpp_provider_context& ypc, _lnav_config* root) {
+                auto* retval = &(root->lc_log_annotations
+                                     .a_definitions[ypc.get_substr_i(0)]);
+
+                return retval;
+            })
+        .with_path_provider<_lnav_config>(
+            [](struct _lnav_config* cfg, std::vector<std::string>& paths_out) {
+                for (const auto& iter : cfg->lc_log_annotations.a_definitions) {
+                    paths_out.emplace_back(iter.first.to_string());
+                }
+            })
+        .with_children(annotation_handlers),
+};
+
+static const struct json_path_container log_date_time_handlers = {
+    yajlpp::property_handler("convert-zoned-to-local")
+        .with_description("Convert timestamps with ")
+        .with_pattern(R"(^[\w\-]+(?!\.lnav)$)")
+        .for_field(&_lnav_config::lc_log_date_time,
+                   &date_time_scanner_ns::config::c_zoned_to_local),
+};
+
 static const struct json_path_container log_source_handlers = {
+    yajlpp::property_handler("date-time")
+        .with_description("Settings related to log message dates and times")
+        .with_children(log_date_time_handlers),
     yajlpp::property_handler("watch-expressions")
         .with_description("Log message watch expressions")
         .with_children(log_source_watch_handlers),
+    yajlpp::property_handler("annotations").with_children(annotations_handlers),
+};
+
+static const struct json_path_container url_scheme_handlers = {
+    yajlpp::property_handler("handler")
+        .with_description(
+            "The name of the lnav script that can handle URLs "
+            "with of this scheme.  This should not include the '.lnav' suffix.")
+        .with_pattern(R"(^[\w\-]+(?!\.lnav)$)")
+        .for_field(&lnav::url_handler::scheme::p_handler),
+};
+
+static const struct json_path_container url_handlers = {
+    yajlpp::pattern_property_handler(R"((?<url_scheme>[a-z][\w\-\+\.]+))")
+        .with_description("Definition of a custom URL scheme")
+        .with_obj_provider<lnav::url_handler::scheme, _lnav_config>(
+            [](const yajlpp_provider_context& ypc, _lnav_config* root) {
+                auto& retval = root->lc_url_handlers
+                                   .c_schemes[ypc.get_substr("url_scheme")];
+                return &retval;
+            })
+        .with_path_provider<_lnav_config>(
+            [](struct _lnav_config* cfg, std::vector<std::string>& paths_out) {
+                for (const auto& iter : cfg->lc_url_handlers.c_schemes) {
+                    paths_out.emplace_back(iter.first);
+                }
+            })
+        .with_children(url_scheme_handlers),
 };
 
 static const struct json_path_container tuning_handlers = {
     yajlpp::property_handler("archive-manager")
         .with_description("Settings related to opening archive files")
         .with_children(archive_handlers),
+    yajlpp::property_handler("piper")
+        .with_description("Settings related to capturing piped data")
+        .with_children(piper_handlers),
     yajlpp::property_handler("file-vtab")
         .with_description("Settings related to the lnav_file virtual-table")
         .with_children(file_vtab_handlers),
@@ -1218,6 +1461,9 @@ static const struct json_path_container tuning_handlers = {
     yajlpp::property_handler("clipboard")
         .with_description("Settings related to the clipboard")
         .with_children(sysclip_handlers),
+    yajlpp::property_handler("url-scheme")
+        .with_description("Settings related to custom URL handling")
+        .with_children(url_handlers),
 };
 
 const char* DEFAULT_CONFIG_SCHEMA
@@ -1249,10 +1495,10 @@ read_id(yajlpp_parse_context* ypc, const unsigned char* str, size_t len)
         }
         ypc->report_error(
             lnav::console::user_message::error(
-                attr_line_t("'")
-                    .append(lnav::roles::symbol(file_id))
+                attr_line_t()
+                    .append_quoted(lnav::roles::symbol(file_id))
                     .append(
-                        "' is not a supported configuration $schema version"))
+                        " is not a supported configuration $schema version"))
                 .with_snippet(ypc->get_snippet())
                 .with_note(notes)
                 .with_help(handler->get_help_text(ypc)));
@@ -1287,6 +1533,8 @@ const json_path_container lnav_config_handlers = json_path_container {
 
 class active_key_map_listener : public lnav_config_listener {
 public:
+    active_key_map_listener() : lnav_config_listener(__FILE__) {}
+
     void reload_config(error_reporter& reporter) override
     {
         lnav_config.lc_active_keymap = lnav_config.lc_ui_keymaps["default"];
@@ -1342,9 +1590,10 @@ load_config_from(_lnav_config& lconfig,
 {
     yajlpp_parse_context ypc(intern_string::lookup(path.string()),
                              &lnav_config_handlers);
-    struct userdata ud(errors);
+    struct config_userdata ud(errors);
     auto_fd fd;
 
+    log_info("loading configuration from %s", path.c_str());
     ypc.ypc_locations = &lnav_config_locations;
     ypc.with_obj(lconfig);
     ypc.ypc_userdata = &ud;
@@ -1358,11 +1607,10 @@ load_config_from(_lnav_config& lconfig,
                     .with_errno_reason());
         }
     } else {
-        auto_mem<yajl_handle_t> handle(yajl_free);
         char buffer[2048];
         ssize_t rc = -1;
 
-        handle = yajl_alloc(&ypc.ypc_callbacks, nullptr, &ypc);
+        auto handle = yajlpp::alloc_handle(&ypc.ypc_callbacks, &ypc);
         yajl_config(handle, yajl_allow_comments, 1);
         yajl_config(handle, yajl_allow_multiple_values, 1);
         ypc.ypc_handle = handle;
@@ -1398,10 +1646,10 @@ load_default_config(struct _lnav_config& config_obj,
 {
     yajlpp_parse_context ypc_builtin(intern_string::lookup(bsf.get_name()),
                                      &lnav_config_handlers);
-    auto_mem<yajl_handle_t> handle(yajl_free);
-    struct userdata ud(errors);
+    struct config_userdata ud(errors);
 
-    handle = yajl_alloc(&ypc_builtin.ypc_callbacks, nullptr, &ypc_builtin);
+    auto handle
+        = yajlpp::alloc_handle(&ypc_builtin.ypc_callbacks, &ypc_builtin);
     ypc_builtin.ypc_locations = &lnav_config_locations;
     ypc_builtin.with_handle(handle);
     ypc_builtin.with_obj(config_obj);
@@ -1415,9 +1663,7 @@ load_default_config(struct _lnav_config& config_obj,
 
     yajl_config(handle, yajl_allow_comments, 1);
     yajl_config(handle, yajl_allow_multiple_values, 1);
-    if (ypc_builtin.parse(bsf.to_string_fragment()) == yajl_status_ok) {
-        ypc_builtin.complete_parse();
-    }
+    ypc_builtin.parse_doc(bsf.to_string_fragment());
 
     return path == "*" || ypc_builtin.ypc_active_paths.empty();
 }
@@ -1462,7 +1708,7 @@ load_config(const std::vector<ghc::filesystem::path>& extra_paths,
         log_info("loading builtin configuration into base");
         load_default_configs(lnav_config, "*", errors);
 
-        log_info("loading user configuration files");
+        log_info("loading installed configuration files");
         for (const auto& extra_path : extra_paths) {
             auto config_path = extra_path / "configs/*/*.json";
             static_root_mem<glob_t, globfree> gl;
@@ -1501,6 +1747,19 @@ load_config(const std::vector<ghc::filesystem::path>& extra_paths,
     rollback_lnav_config = lnav_config;
 }
 
+std::string
+dump_config()
+{
+    yajlpp_gen gen;
+    yajlpp_gen_context ygc(gen, lnav_config_handlers);
+
+    yajl_gen_config(gen, yajl_gen_beautify, true);
+    ygc.with_obj(lnav_config);
+    ygc.gen();
+
+    return gen.to_string_fragment().to_string();
+}
+
 void
 reset_config(const std::string& path)
 {
@@ -1525,14 +1784,12 @@ reset_config(const std::string& path)
         }
 
         if (jph != nullptr && jph->jph_children && jph->jph_obj_deleter) {
-            pcre_context_static<30> pc;
             auto key_start = ypc.ypc_path_index_stack.back();
-            pcre_input pi(&ypc.ypc_path[key_start + 1],
-                          0,
-                          ypc.ypc_path.size() - key_start - 2);
-            yajlpp_provider_context provider_ctx{{pc, pi},
-                                                 static_cast<size_t>(-1)};
-            jph->jph_regex->match(pc, pi);
+            auto path_frag = string_fragment::from_byte_range(
+                ypc.ypc_path.data(), key_start + 1, ypc.ypc_path.size());
+            auto md = jph->jph_regex->create_match_data();
+            yajlpp_provider_context provider_ctx{&md, static_cast<size_t>(-1)};
+            jph->jph_regex->capture_from(path_frag).into(md).matches();
 
             jph->jph_obj_deleter(provider_ctx, ypc.ypc_obj_stack.top());
         }
@@ -1594,31 +1851,41 @@ reload_config(std::vector<lnav::console::user_message>& errors)
     while (curr != nullptr) {
         auto reporter = [&errors](const void* cfg_value,
                                   const lnav::console::user_message& errmsg) {
+            log_error("configuration error: %s",
+                      errmsg.to_attr_line().get_string().c_str());
             auto cb = [&cfg_value, &errors, &errmsg](
                           const json_path_handler_base& jph,
                           const std::string& path,
-                          void* mem) {
+                          const void* mem) {
                 if (mem != cfg_value) {
                     return;
                 }
 
                 auto loc_iter
                     = lnav_config_locations.find(intern_string::lookup(path));
-                if (loc_iter == lnav_config_locations.end()) {
-                    return;
+                auto has_loc = loc_iter != lnav_config_locations.end();
+                auto um = has_loc
+                    ? lnav::console::user_message::error(
+                          attr_line_t()
+                              .append("invalid value for property ")
+                              .append_quoted(lnav::roles::symbol(path)))
+                          .with_reason(errmsg)
+                    : errmsg;
+                um.with_help(jph.get_help_text(path));
+
+                if (has_loc) {
+                    um.with_snippet(
+                        lnav::console::snippet::from(loc_iter->second.sl_source,
+                                                     "")
+                            .with_line(loc_iter->second.sl_line_number));
+                } else {
+                    um.um_message
+                        = attr_line_t()
+                              .append("missing value for property ")
+                              .append_quoted(lnav::roles::symbol(path));
                 }
 
-                errors.emplace_back(
-                    lnav::console::user_message::error(
-                        attr_line_t()
-                            .append("invalid value for property ")
-                            .append_quoted(lnav::roles::symbol(path)))
-                        .with_reason(errmsg)
-                        .with_snippet(
-                            lnav::console::snippet::from(
-                                loc_iter->second.sl_source, "")
-                                .with_line(loc_iter->second.sl_line_number))
-                        .with_help(jph.get_help_text(path)));
+                errors.emplace_back(um);
             };
 
             for (const auto& jph : lnav_config_handlers.jpc_children) {
