@@ -528,6 +528,9 @@ plain_text_source::adjacent_anchor(vis_line_t vl, text_anchors::direction dir)
                 if (neighbors_res->cnr_next) {
                     return this->line_for_offset(
                         neighbors_res->cnr_next.value()->hn_start);
+                } else if (!md.m_sections_root->hn_children.empty()) {
+                    return this->line_for_offset(
+                        md.m_sections_root->hn_children[0]->hn_start);
                 }
                 break;
             }
