@@ -82,7 +82,7 @@ public:
         return this->fss_time_delegate->row_for_time(time_bucket);
     }
 
-    nonstd::optional<struct timeval> time_for_row(vis_line_t row) override
+    nonstd::optional<row_info> time_for_row(vis_line_t row) override
     {
         return this->fss_lines | lnav::itertools::nth(row)
             | lnav::itertools::flat_map([this](const auto row) {
@@ -90,17 +90,14 @@ public:
                });
     }
 
-    bool list_value_for_overlay(const listview_curses& lv,
-                                int y,
-                                int bottom,
+    void list_value_for_overlay(const listview_curses& lv,
                                 vis_line_t line,
-                                attr_line_t& value_out) override
+                                std::vector<attr_line_t>& value_out) override
     {
         if (this->fss_overlay_delegate != nullptr) {
-            return this->fss_overlay_delegate->list_value_for_overlay(
-                lv, y, bottom, line, value_out);
+            this->fss_overlay_delegate->list_value_for_overlay(
+                lv, line, value_out);
         }
-        return false;
     }
 
     text_sub_source* fss_delegate;
@@ -301,6 +298,7 @@ log_spectro_value_source::spectro_mark(textview_curses& tc,
 
         values.clear();
         lf->read_full_message(ll, values.lvv_sbr);
+        values.lvv_sbr.erase_ansi();
         sa.clear();
         format->annotate(cl, sa, values, false);
 
