@@ -33,21 +33,12 @@
 
 #include "config.h"
 
-static std::shared_ptr<pcrepp>
-xpcre_compile(const char* pattern, int options = 0)
+template<typename T, std::size_t N>
+static std::shared_ptr<lnav::pcre2pp::code>
+xpcre_compile(const T (&pattern)[N], int options = 0)
 {
-    auto compile_res = pcrepp::shared_from_str(pattern, options);
-
-    if (compile_res.isErr()) {
-        auto ce = compile_res.unwrapErr();
-
-        fprintf(stderr, "internal error: failed to compile -- %s\n", pattern);
-        fprintf(stderr, "internal error: %s\n", ce.ce_msg);
-
-        exit(1);
-    }
-
-    return compile_res.unwrap();
+    return lnav::pcre2pp::code::from_const(pattern, options | PCRE2_MULTILINE)
+        .to_shared();
 }
 
 void
@@ -103,6 +94,7 @@ setup_highlights(highlight_map_t& hm)
                                     "\\bconst\\b|"
                                     "\\bcontinue\\b|"
                                     "\\bcrate\\b|"
+                                    "\\bdyn\\b|"
                                     "\\belse\\b|"
                                     "\\bif\\b|"
                                     "\\bif let\\b|"
@@ -174,8 +166,10 @@ setup_highlights(highlight_map_t& hm)
                                     "\\bchar\\b|"
                                     "\\bclass\\b|"
                                     "\\bconst\\b|"
+                                    "\\bconstexpr\\b|"
                                     "\\bconst_cast\\b|"
                                     "\\bcontinue\\b|"
+                                    "\\bdecltype\\b|"
                                     "\\bdefault\\b|"
                                     "\\bdelete\\b|"
                                     "\\bdo\\b|"
@@ -183,6 +177,7 @@ setup_highlights(highlight_map_t& hm)
                                     "\\bdynamic_cast\\b|"
                                     "\\belse\\b|"
                                     "\\benum\\b|"
+                                    "\\bexplicit\\b|"
                                     "\\bextends\\b|"
                                     "\\bextern\\b|"
                                     "\\bfalse\\b|"
@@ -204,6 +199,8 @@ setup_highlights(highlight_map_t& hm)
                                     "\\bnamespace\\b|"
                                     "\\bnative\\b|"
                                     "\\bnew\\b|"
+                                    "\\bnoexcept\\b|"
+                                    "\\bnullptr\\b|"
                                     "\\boperator\\b|"
                                     "\\bpackage\\b|"
                                     "\\bprivate\\b|"
@@ -224,6 +221,7 @@ setup_highlights(highlight_map_t& hm)
                                     "\\bsynchronized\\b|"
                                     "\\btemplate\\b|"
                                     "\\bthis\\b|"
+                                    "\\bthread_local\\b|"
                                     "\\bthrow\\b|"
                                     "\\bthrows\\b|"
                                     "\\btransient\\b|"
@@ -231,6 +229,7 @@ setup_highlights(highlight_map_t& hm)
                                     "\\btrue\\b|"
                                     "\\btypedef\\b|"
                                     "\\btypeid\\b|"
+                                    "\\btypename\\b|"
                                     "\\bunion\\b|"
                                     "\\bunsigned\\b|"
                                     "\\busing\\b|"
@@ -245,6 +244,16 @@ setup_highlights(highlight_map_t& hm)
               .with_text_format(text_format_t::TF_JAVA)
               .with_role(role_t::VCR_KEYWORD);
 
+    hm[{highlight_source_t::INTERNAL, "json.keyword"}]
+        = highlighter(xpcre_compile(R"((?:null|true|false))"))
+              .with_nestable(false)
+              .with_text_format(text_format_t::TF_JSON)
+              .with_role(role_t::VCR_KEYWORD);
+    hm[{highlight_source_t::INTERNAL, "json.number"}]
+        = highlighter(xpcre_compile(R"(-?\d+(?:\.\d+(?:[eE][+\-]?\d+)?)?)"))
+              .with_nestable(false)
+              .with_text_format(text_format_t::TF_JSON)
+              .with_role(role_t::VCR_NUMBER);
     hm[{highlight_source_t::INTERNAL, "sql.0.comment"}]
         = highlighter(xpcre_compile("(?:(?<=[\\s;])|^)--.*"))
               .with_text_format(text_format_t::TF_SQL)
@@ -382,7 +391,7 @@ setup_highlights(highlight_map_t& hm)
                                     "\\bWITH\\b|"
                                     "\\bWITHOUT\\b"
                                     ")",
-                                    PCRE_CASELESS))
+                                    PCRE2_CASELESS))
               .with_nestable(false)
               .with_text_format(text_format_t::TF_SQL)
               .with_role(role_t::VCR_KEYWORD);
@@ -395,12 +404,23 @@ setup_highlights(highlight_map_t& hm)
               .with_role(role_t::VCR_FILE);
     hm[{highlight_source_t::INTERNAL, "1.stringd"}]
         = highlighter(xpcre_compile(R"("(?:\\.|[^"])*")"))
+              .with_nestable(false)
               .with_role(role_t::VCR_STRING);
     hm[{highlight_source_t::INTERNAL, "1.strings"}]
         = highlighter(xpcre_compile(R"((?<![A-WY-Za-qstv-z])'(?:\\.|[^'])*')"))
+              .with_nestable(false)
+              .with_text_format(text_format_t::TF_C_LIKE)
+              .with_text_format(text_format_t::TF_JAVA)
+              .with_text_format(text_format_t::TF_MARKDOWN)
+              .with_text_format(text_format_t::TF_PYTHON)
+              .with_text_format(text_format_t::TF_SQL)
+              .with_text_format(text_format_t::TF_XML)
+              .with_text_format(text_format_t::TF_YAML)
+              .with_text_format(text_format_t::TF_TOML)
               .with_role(role_t::VCR_STRING);
     hm[{highlight_source_t::INTERNAL, "1.stringb"}]
         = highlighter(xpcre_compile("`(?:\\\\.|[^`])*`"))
+              .with_nestable(false)
               .with_role(role_t::VCR_STRING);
     hm[{highlight_source_t::INTERNAL, "diffp"}]
         = highlighter(xpcre_compile("^\\+.*")).with_role(role_t::VCR_DIFF_ADD);
@@ -413,7 +433,12 @@ setup_highlights(highlight_map_t& hm)
     hm[{highlight_source_t::INTERNAL, "0.comment"}]
         = highlighter(
               xpcre_compile(
-                  R"((?<=[\s;])//.*|/\*.*\*/|\(\*.*\*\)|^#.*|\s+#.*|dnl.*)"))
+                  R"((?<=[\s;]|^)//.*|/\*.*\*/|\(\*.*\*\)|^\s*#(?!\s*(?:include|if|ifndef|elif|else|endif|error|pragma|define|undef)\b).*|dnl.*)"))
+              .with_nestable(false)
+              .with_role(role_t::VCR_COMMENT);
+    hm[{highlight_source_t::INTERNAL, "z.comment"}]
+        = highlighter(xpcre_compile(R"(\s+#.*)"))
+              .with_nestable(false)
               .with_role(role_t::VCR_COMMENT);
     hm[{highlight_source_t::INTERNAL, "javadoc"}]
         = highlighter(
@@ -428,6 +453,12 @@ setup_highlights(highlight_map_t& hm)
                             "(?<!\\$)\\$\\((\\w+)\\)|"
                             "(?<!\\$)\\$\\{(\\w+)\\}"
                             ")"))
+              .with_nestable(false)
+              .with_role(role_t::VCR_VARIABLE);
+    hm[{highlight_source_t::INTERNAL, "yaml.var"}]
+        = highlighter(xpcre_compile("^\\s*(?:- )?[a-zA-Z_\\-]+:(?:\\s+|$)"))
+              .with_nestable(false)
+              .with_text_format(text_format_t::TF_YAML)
               .with_role(role_t::VCR_VARIABLE);
     hm[{highlight_source_t::INTERNAL, "rust.sym"}]
         = highlighter(xpcre_compile("\\b[A-Z_][A-Z0-9_]+\\b"))
@@ -445,10 +476,93 @@ setup_highlights(highlight_map_t& hm)
               .with_text_format(text_format_t::TF_C_LIKE)
               .with_text_format(text_format_t::TF_JAVA)
               .with_role(role_t::VCR_SYMBOL);
+    hm[{highlight_source_t::INTERNAL, "cpp"}]
+        = highlighter(
+              xpcre_compile(
+                  R"(^#\s*(?:include|ifdef|ifndef|if|else|elif|error|endif|define|undef|pragma)\b)"))
+              .with_nestable(false)
+              .with_text_format(text_format_t::TF_C_LIKE)
+              .with_text_format(text_format_t::TF_JAVA)
+              .with_role(role_t::VCR_KEYWORD);
+    hm[{highlight_source_t::INTERNAL, "shell"}]
+        = highlighter(xpcre_compile("(?:"
+                                    "\\bbreak\\b|"
+                                    "\\bcase\\b|"
+                                    "\\bcd\\b|"
+                                    "\\bcontinue\\b|"
+                                    "\\bdeclare\\b|"
+                                    "\\bdefault\\b|"
+                                    "\\bdo\\b|"
+                                    "\\bdone\\b|"
+                                    "\\becho\\b|"
+                                    "\\belif\\b|"
+                                    "\\belse\\b|"
+                                    "\\besac\\b|"
+                                    "\\beval\\b|"
+                                    "\\bexit\\b|"
+                                    "\\bexport\\b|"
+                                    "\\bfalse\\b|"
+                                    "\\bfi\\b|"
+                                    "\\bfor\\b|"
+                                    "\\bfunction\\b|"
+                                    "\\bif\\b|"
+                                    "\\bin\\b|"
+                                    "\\blocal\\b|"
+                                    "\\bprintf\\b|"
+                                    "\\bpwd\\b|"
+                                    "\\bread\\b|"
+                                    "\\breadonly\\b|"
+                                    "\\breturn\\b|"
+                                    "\\bset\\b|"
+                                    "\\bshift\\b|"
+                                    "\\bsource\\b|"
+                                    "\\btest\\b|"
+                                    "\\bthen\\b|"
+                                    "\\btrap\\b|"
+                                    "\\btrue\\b|"
+                                    "\\bunset\\b|"
+                                    "\\bunsetenv\\b|"
+                                    "\\buntil\\b|"
+                                    "\\bwhich\\b|"
+                                    "\\bwhile\\b"
+                                    ")"))
+              .with_nestable(false)
+              .with_text_format(text_format_t::TF_SHELL_SCRIPT)
+              .with_role(role_t::VCR_KEYWORD);
     hm[{highlight_source_t::INTERNAL, "num"}]
         = highlighter(xpcre_compile(R"(\b-?(?:\d+|0x[a-zA-Z0-9]+)\b)"))
               .with_nestable(false)
               .with_text_format(text_format_t::TF_C_LIKE)
               .with_text_format(text_format_t::TF_JAVA)
+              .with_text_format(text_format_t::TF_YAML)
               .with_role(role_t::VCR_NUMBER);
+    hm[{highlight_source_t::INTERNAL, "fun"}]
+        = highlighter(xpcre_compile(R"((\w+)\()"))
+              .with_nestable(false)
+              .with_text_format(text_format_t::TF_C_LIKE)
+              .with_text_format(text_format_t::TF_JAVA)
+              .with_text_format(text_format_t::TF_PYTHON)
+              .with_text_format(text_format_t::TF_RUST)
+              .with_text_format(text_format_t::TF_SQL)
+              .with_role(role_t::VCR_FUNCTION);
+    hm[{highlight_source_t::INTERNAL, "sep"}]
+        = highlighter(xpcre_compile(R"(\.|\s+&(?=\w)|(?<=\w)&\s+|::|\%\b)"))
+              .with_nestable(false)
+              .with_text_format(text_format_t::TF_C_LIKE)
+              .with_text_format(text_format_t::TF_JAVA)
+              .with_text_format(text_format_t::TF_PYTHON)
+              .with_text_format(text_format_t::TF_RUST)
+              .with_text_format(text_format_t::TF_SQL)
+              .with_role(role_t::VCR_SEP_REF_ACC);
+    hm[{highlight_source_t::INTERNAL, "type"}]
+        = highlighter(
+              xpcre_compile(
+                  R"(\b(class|struct|enum(?:\s+class)?)\s+(\w+)\b|\b(\w+_t)\b)"))
+              .with_nestable(false)
+              .with_text_format(text_format_t::TF_C_LIKE)
+              .with_text_format(text_format_t::TF_JAVA)
+              .with_text_format(text_format_t::TF_PYTHON)
+              .with_text_format(text_format_t::TF_RUST)
+              .with_text_format(text_format_t::TF_SQL)
+              .with_role(role_t::VCR_TYPE);
 }
