@@ -33,13 +33,13 @@
 #include "textview_curses.hh"
 
 #include "base/ansi_scrubber.hh"
+#include "base/humanize.time.hh"
 #include "base/injector.hh"
 #include "base/time_util.hh"
 #include "config.h"
-#include "data_parser.hh"
 #include "fmt/format.h"
 #include "lnav_config.hh"
-#include "log_format.hh"
+#include "log_format_fwd.hh"
 #include "logfile.hh"
 #include "shlex.hh"
 #include "view_curses.hh"
@@ -79,9 +79,9 @@ text_filter::revert_to_last(logfile_filter_state& lfs, size_t rollback_size)
 void
 text_filter::add_line(logfile_filter_state& lfs,
                       logfile::const_iterator ll,
-                      shared_buffer_ref& line)
+                      const shared_buffer_ref& line)
 {
-    bool match_state = this->matches(*lfs.tfs_logfile, ll, line);
+    bool match_state = this->matches(line_source{*lfs.tfs_logfile, ll}, line);
 
     if (ll->is_message()) {
         this->end_of_message(lfs);
@@ -124,12 +124,66 @@ text_filter::end_of_message(logfile_filter_state& lfs)
     lfs.tfs_lines_for_message[this->lf_index] = 0;
 }
 
+log_accel::direction_t
+text_accel_source::get_line_accel_direction(vis_line_t vl)
+{
+    log_accel la;
+
+    while (vl >= 0) {
+        const auto* curr_line = this->text_accel_get_line(vl);
+
+        if (!curr_line->is_message()) {
+            --vl;
+            continue;
+        }
+
+        if (!la.add_point(curr_line->get_time_in_millis())) {
+            break;
+        }
+
+        --vl;
+    }
+
+    return la.get_direction();
+}
+
+std::string
+text_accel_source::get_time_offset_for_line(textview_curses& tc, vis_line_t vl)
+{
+    auto ll = this->text_accel_get_line(vl);
+    auto curr_tv = ll->get_timeval();
+    struct timeval diff_tv;
+
+    auto prev_umark = tc.get_bookmarks()[&textview_curses::BM_USER].prev(vl);
+    auto next_umark = tc.get_bookmarks()[&textview_curses::BM_USER].next(vl);
+    auto prev_emark
+        = tc.get_bookmarks()[&textview_curses::BM_USER_EXPR].prev(vl);
+    auto next_emark
+        = tc.get_bookmarks()[&textview_curses::BM_USER_EXPR].next(vl);
+    if (!prev_umark && !prev_emark && (next_umark || next_emark)) {
+        auto next_line = this->text_accel_get_line(
+            std::max(next_umark.value_or(0), next_emark.value_or(0)));
+
+        diff_tv = curr_tv - next_line->get_timeval();
+    } else {
+        auto prev_row
+            = std::max(prev_umark.value_or(0), prev_emark.value_or(0));
+        auto first_line = this->text_accel_get_line(prev_row);
+        auto start_tv = first_line->get_timeval();
+        diff_tv = curr_tv - start_tv;
+    }
+
+    return humanize::time::duration::from_tv(diff_tv).to_string();
+}
+
 const bookmark_type_t textview_curses::BM_USER("user");
 const bookmark_type_t textview_curses::BM_USER_EXPR("user-expr");
 const bookmark_type_t textview_curses::BM_SEARCH("search");
 const bookmark_type_t textview_curses::BM_META("meta");
+const bookmark_type_t textview_curses::BM_PARTITION("partition");
 
-textview_curses::textview_curses() : tc_search_action(noop_func{})
+textview_curses::textview_curses()
+    : lnav_config_listener(__FILE__), tc_search_action(noop_func{})
 {
     this->set_data_source(this);
 }
@@ -155,11 +209,13 @@ textview_curses::reload_config(error_reporter& reporter)
         iter = this->tc_highlights.erase(iter);
     }
 
-    std::map<std::string, std::string> vars;
+    std::map<std::string, scoped_value_t> vars;
     auto curr_theme_iter
         = lnav_config.lc_ui_theme_defs.find(lnav_config.lc_ui_theme);
     if (curr_theme_iter != lnav_config.lc_ui_theme_defs.end()) {
-        vars = curr_theme_iter->second.lt_vars;
+        for (const auto& vpair : curr_theme_iter->second.lt_vars) {
+            vars[vpair.first] = vpair.second;
+        }
     }
 
     for (const auto& theme_name : {DEFAULT_THEME_NAME, lnav_config.lc_ui_theme})
@@ -171,19 +227,7 @@ textview_curses::reload_config(error_reporter& reporter)
         }
 
         for (const auto& hl_pair : theme_iter->second.lt_highlights) {
-            if (hl_pair.second.hc_regex.empty()) {
-                continue;
-            }
-
-            auto regex = pcrepp::shared_from_str(hl_pair.second.hc_regex);
-
-            if (regex.isErr()) {
-                auto ce = regex.unwrapErr();
-                reporter(&hl_pair.second.hc_regex,
-                         lnav::console::user_message::error(fmt::format(
-                             FMT_STRING("invalid highlight regex: {} at {}"),
-                             ce.ce_msg,
-                             ce.ce_offset)));
+            if (hl_pair.second.hc_regex.pp_value == nullptr) {
                 continue;
             }
 
@@ -194,8 +238,8 @@ textview_curses::reload_config(error_reporter& reporter)
 
             fg1 = sc.sc_color;
             bg1 = sc.sc_background_color;
-            shlex(fg1).eval(fg_color, vars);
-            shlex(bg1).eval(bg_color, vars);
+            shlex(fg1).eval(fg_color, scoped_resolver{&vars});
+            shlex(bg1).eval(bg_color, scoped_resolver{&vars});
 
             auto fg = styling::color_unit::from_str(fg_color).unwrapOrElse(
                 [&](const auto& msg) {
@@ -228,12 +272,16 @@ textview_curses::reload_config(error_reporter& reporter)
                 attrs.ta_attrs |= A_UNDERLINE;
             }
             this->tc_highlights[{highlight_source_t::THEME, hl_pair.first}]
-                = highlighter(regex.unwrap())
+                = highlighter(hl_pair.second.hc_regex.pp_value)
                       .with_attrs(attrs)
                       .with_color(fg, bg)
                       .with_nestable(false);
         }
     }
+
+    if (this->tc_reload_config_delegate) {
+        this->tc_reload_config_delegate(*this);
+    }
 }
 
 void
@@ -287,7 +335,7 @@ void
 textview_curses::grep_end_batch(grep_proc<vis_line_t>& gp)
 {
     if (this->tc_follow_deadline.tv_sec
-        && this->tc_follow_top == this->get_top())
+        && this->tc_follow_selection == this->get_selection())
     {
         struct timeval now;
 
@@ -437,8 +485,11 @@ textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
     this->tc_sub_source->text_value_for_line(*this, row, str);
     this->tc_sub_source->text_attrs_for_line(*this, row, sa);
 
-    scrub_ansi_string(str, &sa);
+    for (const auto& attr : sa) {
+        require_ge(attr.sa_range.lr_start, 0);
+    }
 
+    scrub_ansi_string(str, &sa);
     struct line_range body, orig_line;
 
     body = find_string_attr_range(sa, &SA_BODY);
@@ -458,6 +509,17 @@ textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
         format_name = format_attr_opt.value().get();
     }
 
+    if (this->is_selectable() && row == this->get_selection()
+        && this->tc_cursor_role && this->tc_disabled_cursor_role)
+    {
+        auto role = this->get_overlay_selection()
+            ? this->tc_disabled_cursor_role.value()
+            : this->tc_cursor_role.value();
+
+        sa.emplace_back(line_range{orig_line.lr_start, -1},
+                        VC_ROLE.value(role));
+    }
+
     for (auto& tc_highlight : this->tc_highlights) {
         bool internal_hl
             = tc_highlight.first.first == highlight_source_t::INTERNAL
@@ -492,61 +554,11 @@ textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
         value_out.apply_hide();
     }
 
-#if 0
-    typedef std::map<std::string, role_t> key_map_t;
-    static key_map_t key_roles;
-
-    data_scanner ds(str);
-    data_parser  dp(&ds);
-
-    dp.parse();
-
-    for (list<data_parser::element>::iterator iter = dp.dp_stack.begin();
-         iter != dp.dp_stack.end();
-         ++iter) {
-        view_colors &vc = view_colors::singleton();
-
-        if (iter->e_token == DNT_PAIR) {
-            list<data_parser::element>::iterator pair_iter;
-            key_map_t::iterator km_iter;
-            data_token_t        value_token;
-            struct line_range   lr;
-            string key;
-
-            value_token =
-                iter->e_sub_elements->back().e_sub_elements->front().e_token;
-            if (value_token == DT_STRING) {
-                continue;
-            }
-
-            lr.lr_start = iter->e_capture.c_begin;
-            lr.lr_end   = iter->e_capture.c_end;
-
-            key = ds.get_input().get_substr(
-                &iter->e_sub_elements->front().e_capture);
-            if ((km_iter = key_roles.find(key)) == key_roles.end()) {
-                key_roles[key] = vc.next_highlight();
-            }
-            /* fprintf(stderr, "key = %s\n", key.c_str()); */
-            sa[lr].insert(make_string_attr("style",
-                                           vc.attrs_for_role(key_roles[key])));
-
-            pair_iter = iter->e_sub_elements->begin();
-            ++pair_iter;
-
-            lr.lr_start = pair_iter->e_capture.c_begin;
-            lr.lr_end   = pair_iter->e_capture.c_end;
-            sa[lr].insert(make_string_attr("style",
-                                           COLOR_PAIR(view_colors::VC_WHITE) |
-                                           A_BOLD));
-        }
-    }
-#endif
-
     const auto& user_marks = this->tc_bookmarks[&BM_USER];
     const auto& user_expr_marks = this->tc_bookmarks[&BM_USER_EXPR];
-    if (binary_search(user_marks.begin(), user_marks.end(), row)
-        || binary_search(user_expr_marks.begin(), user_expr_marks.end(), row))
+    if (std::binary_search(user_marks.begin(), user_marks.end(), row)
+        || std::binary_search(
+            user_expr_marks.begin(), user_expr_marks.end(), row))
     {
         sa.emplace_back(line_range{orig_line.lr_start, -1},
                         VC_STYLE.value(text_attrs{A_REVERSE}));
@@ -557,7 +569,7 @@ void
 textview_curses::execute_search(const std::string& regex_orig)
 {
     std::string regex = regex_orig;
-    std::shared_ptr<pcrepp> code;
+    std::shared_ptr<lnav::pcre2pp::code> code;
 
     if ((this->tc_search_child == nullptr)
         || (regex != this->tc_current_search))
@@ -571,27 +583,26 @@ textview_curses::execute_search(const std::string& regex_orig)
 
         if (regex.empty()) {
         } else {
-            auto compile_res
-                = pcrepp::shared_from_str(regex, PCRE_CASELESS | PCRE_UTF8);
+            auto compile_res = lnav::pcre2pp::code::from(regex, PCRE2_CASELESS);
 
             if (compile_res.isErr()) {
                 auto ce = compile_res.unwrapErr();
-                regex = pcrepp::quote(regex);
+                regex = lnav::pcre2pp::quote(regex);
 
                 log_info("invalid search regex (%s), using quoted: %s",
-                         ce.ce_msg,
+                         ce.get_message().c_str(),
                          regex.c_str());
 
                 auto compile_quote_res
-                    = pcrepp::shared_from_str(regex, PCRE_CASELESS | PCRE_UTF8);
+                    = lnav::pcre2pp::code::from(regex, PCRE2_CASELESS);
                 if (compile_quote_res.isErr()) {
                     log_error("Unable to compile quoted regex: %s",
                               regex.c_str());
                 } else {
-                    code = compile_quote_res.unwrap();
+                    code = compile_quote_res.unwrap().to_shared();
                 }
             } else {
-                code = compile_res.unwrap();
+                code = compile_res.unwrap().to_shared();
             }
         }
 
@@ -604,7 +615,7 @@ textview_curses::execute_search(const std::string& regex_orig)
             hm[{highlight_source_t::PREVIEW, "search"}] = hl;
 
             auto gp = injector::get<std::shared_ptr<grep_proc<vis_line_t>>>(
-                code->p_code, *this);
+                code, *this);
 
             gp->set_sink(this);
             auto top = this->get_top();
@@ -626,7 +637,7 @@ textview_curses::execute_search(const std::string& regex_orig)
                 this->tc_sub_source->get_grepper() | [this, code](auto pair) {
                     auto sgp
                         = injector::get<std::shared_ptr<grep_proc<vis_line_t>>>(
-                            code->p_code, *pair.first);
+                            code, *pair.first);
 
                     sgp->set_sink(pair.second);
                     sgp->queue_request(0_vl);
@@ -661,15 +672,15 @@ textview_curses::horiz_shift(vis_line_t start, vis_line_t end, int off_start)
         this->listview_value_for_rows(*this, start, rows);
 
         const auto& str = rows[0].get_string();
-        pcre_context_static<60> pc;
-        pcre_input pi(str);
-        while (hl_iter->second.h_regex->match(pc, pi)) {
-            if (pc.all()->c_begin < off_start) {
-                prev_hit = std::max(prev_hit, pc.all()->c_begin);
-            } else if (pc.all()->c_begin > off_start) {
-                next_hit = std::min(next_hit, pc.all()->c_begin);
-            }
-        }
+        hl_iter->second.h_regex->capture_from(str).for_each(
+            [&](lnav::pcre2pp::match_data& md) {
+                auto cap = md[0].value();
+                if (cap.sf_begin < off_start) {
+                    prev_hit = std::max(prev_hit, cap.sf_begin);
+                } else if (cap.sf_begin > off_start) {
+                    next_hit = std::min(next_hit, cap.sf_begin);
+                }
+            });
     }
 
     if (prev_hit == -1 && next_hit == INT_MAX) {
@@ -813,11 +824,21 @@ textview_curses::grep_value_for_line(vis_line_t line, std::string& value_out)
 }
 
 void
-text_time_translator::scroll_invoked(textview_curses* tc)
+text_sub_source::scroll_invoked(textview_curses* tc)
 {
-    if (tc->get_inner_height() > 0) {
-        this->time_for_row(tc->get_top()) |
-            [this](auto new_top_time) { this->ttt_top_time = new_top_time; };
+    auto* ttt = dynamic_cast<text_time_translator*>(this);
+
+    if (ttt != nullptr) {
+        ttt->ttt_scroll_invoked(tc);
+    }
+}
+
+void
+text_time_translator::ttt_scroll_invoked(textview_curses* tc)
+{
+    if (tc->get_inner_height() > 0 && tc->get_selection() >= 0_vl) {
+        this->time_for_row(tc->get_selection()) |
+            [this](auto new_top_ri) { this->ttt_top_row_info = new_top_ri; };
     }
 }
 
@@ -825,34 +846,20 @@ void
 text_time_translator::data_reloaded(textview_curses* tc)
 {
     if (tc->get_inner_height() == 0) {
+        this->ttt_top_row_info = nonstd::nullopt;
         return;
     }
-    if (tc->get_top() > tc->get_inner_height()) {
-        if (this->ttt_top_time.tv_sec != 0) {
-            this->row_for_time(this->ttt_top_time) |
-                [tc](auto new_top) { tc->set_top(new_top); };
-        }
-        return;
+    if (this->ttt_top_row_info) {
+        this->row_for(this->ttt_top_row_info.value()) |
+            [tc](auto new_top) { tc->set_selection(new_top); };
     }
-    this->time_for_row(tc->get_top()) | [this, tc](auto top_time) {
-        if (top_time != this->ttt_top_time) {
-            if (this->ttt_top_time.tv_sec != 0) {
-                this->row_for_time(this->ttt_top_time) |
-                    [tc](auto new_top) { tc->set_top(new_top); };
-            }
-            this->time_for_row(tc->get_top()) | [this](auto new_top_time) {
-                this->ttt_top_time = new_top_time;
-            };
-        }
-    };
 }
 
 template class bookmark_vector<vis_line_t>;
 
 bool
-empty_filter::matches(const logfile& lf,
-                      logfile::const_iterator ll,
-                      shared_buffer_ref& line)
+empty_filter::matches(nonstd::optional<line_source> ls,
+                      const shared_buffer_ref& line)
 {
     return false;
 }
@@ -1127,8 +1134,7 @@ logfile_filter_state::content_line_to_vis_line(uint32_t line)
 std::string
 text_anchors::to_anchor_string(const std::string& raw)
 {
-    static const pcrepp ANCHOR_RE(R"([^\w]+)");
+    static const auto ANCHOR_RE = lnav::pcre2pp::code::from_const(R"([^\w]+)");
 
-    return fmt::format(FMT_STRING("#{}"),
-                       ANCHOR_RE.replace(tolower(raw).c_str(), "-"));
+    return fmt::format(FMT_STRING("#{}"), ANCHOR_RE.replace(tolower(raw), "-"));
 }
