@@ -34,8 +34,6 @@
 
 #include "yajlpp.hh"
 
-#include "base/fs_util.hh"
-#include "base/snippet_highlighters.hh"
 #include "config.h"
 #include "fmt/format.h"
 #include "ghc/filesystem.hpp"
@@ -145,8 +143,10 @@ json_path_handler_base::json_path_handler_base(const std::string& property)
     : jph_property(property.back() == '#'
                        ? property.substr(0, property.size() - 1)
                        : property),
-      jph_regex(
-          std::make_shared<pcrepp>(pcrepp::quote(property), PCRE_ANCHORED)),
+      jph_regex(lnav::pcre2pp::code::from(lnav::pcre2pp::quote(property),
+                                          PCRE2_ANCHORED)
+                    .unwrap()
+                    .to_shared()),
       jph_is_array(property.back() == '#')
 {
     memset(&this->jph_callbacks, 0, sizeof(this->jph_callbacks));
@@ -160,28 +160,20 @@ scrub_pattern(const std::string& pattern)
     return std::regex_replace(pattern, CAPTURE, "(");
 }
 
-json_path_handler_base::json_path_handler_base(const pcrepp& property)
-    : jph_property(scrub_pattern(property.p_pattern)),
-      jph_regex(std::make_shared<pcrepp>(property)),
-      jph_is_array(property.p_pattern.back() == '#'),
-      jph_is_pattern_property(true)
-{
-    memset(&this->jph_callbacks, 0, sizeof(this->jph_callbacks));
-}
-
-json_path_handler_base::json_path_handler_base(std::string property,
-                                               const pcrepp& property_re)
-    : jph_property(std::move(property)),
-      jph_regex(std::make_shared<pcrepp>(property_re)),
-      jph_is_array(property_re.p_pattern.find('#') != std::string::npos)
+json_path_handler_base::json_path_handler_base(
+    const std::shared_ptr<const lnav::pcre2pp::code>& property)
+    : jph_property(scrub_pattern(property->get_pattern())), jph_regex(property),
+      jph_is_array(property->get_pattern().find('#') != std::string::npos),
+      jph_is_pattern_property(property->get_capture_count() > 0)
 {
     memset(&this->jph_callbacks, 0, sizeof(this->jph_callbacks));
 }
 
 json_path_handler_base::json_path_handler_base(
-    std::string property, const std::shared_ptr<pcrepp>& property_re)
+    std::string property,
+    const std::shared_ptr<const lnav::pcre2pp::code>& property_re)
     : jph_property(std::move(property)), jph_regex(property_re),
-      jph_is_array(property_re->p_pattern.find('#') != std::string::npos)
+      jph_is_array(property_re->get_pattern().find('#') != std::string::npos)
 {
     memset(&this->jph_callbacks, 0, sizeof(this->jph_callbacks));
 }
@@ -191,14 +183,12 @@ json_path_handler_base::gen(yajlpp_gen_context& ygc, yajl_gen handle) const
 {
     if (this->jph_is_array) {
         auto size = this->jph_size_provider(ygc.ygc_obj_stack.top());
+        auto md = lnav::pcre2pp::match_data::unitialized();
 
         yajl_gen_string(handle, this->jph_property);
         yajl_gen_array_open(handle);
         for (size_t index = 0; index < size; index++) {
-            pcre_context_static<30> pc;
-            pcre_input pi("");
-
-            yajlpp_provider_context ypc{{pc, pi}, index};
+            yajlpp_provider_context ypc{&md, index};
             yajlpp_gen_context elem_ygc(handle, *this->jph_children);
             elem_ygc.ygc_depth = 1;
             elem_ygc.ygc_obj_stack.push(
@@ -221,10 +211,7 @@ json_path_handler_base::gen(yajlpp_gen_context& ygc, yajl_gen handle) const
 
     if (this->jph_children) {
         for (const auto& lpath : local_paths) {
-            std::string full_path = lpath;
-            if (this->jph_path_provider) {
-                full_path += "/";
-            }
+            std::string full_path = json_ptr::encode_str(lpath);
             int start_depth = ygc.ygc_depth;
 
             yajl_gen_string(handle, lpath);
@@ -232,16 +219,19 @@ json_path_handler_base::gen(yajlpp_gen_context& ygc, yajl_gen handle) const
             ygc.ygc_depth += 1;
 
             if (this->jph_obj_provider) {
-                pcre_context_static<30> pc;
-                pcre_input pi(full_path);
+                static thread_local auto md
+                    = lnav::pcre2pp::match_data::unitialized();
+
+                auto find_res = this->jph_regex->capture_from(full_path)
+                                    .into(md)
+                                    .matches();
 
-                this->jph_regex->match(pc, pi);
                 ygc.ygc_obj_stack.push(this->jph_obj_provider(
-                    {{pc, pi}, yajlpp_provider_context::nindex},
+                    {&md, yajlpp_provider_context::nindex},
                     ygc.ygc_obj_stack.top()));
                 if (!ygc.ygc_default_stack.empty()) {
                     ygc.ygc_default_stack.push(this->jph_obj_provider(
-                        {{pc, pi}, yajlpp_provider_context::nindex},
+                        {&md, yajlpp_provider_context::nindex},
                         ygc.ygc_default_stack.top()));
                 }
             }
@@ -301,8 +291,9 @@ json_path_handler_base::gen_schema(yajlpp_gen_context& ygc) const
                 schema.gen(this->jph_description);
             }
             if (this->jph_is_pattern_property) {
-                ygc.ygc_path.emplace_back(fmt::format(
-                    FMT_STRING("<{}>"), this->jph_regex->name_for_capture(0)));
+                ygc.ygc_path.emplace_back(
+                    fmt::format(FMT_STRING("<{}>"),
+                                this->jph_regex->get_name_for_capture(1)));
             } else {
                 ygc.ygc_path.emplace_back(this->jph_property);
             }
@@ -312,7 +303,7 @@ json_path_handler_base::gen_schema(yajlpp_gen_context& ygc) const
                                        fmt::join(ygc.ygc_path, "/")));
                 schema.gen("type");
                 if (this->jph_is_array) {
-                    if (this->jph_regex->p_pattern.find("#?")
+                    if (this->jph_regex->get_pattern().find("#?")
                         == std::string::npos)
                     {
                         schema.gen("array");
@@ -349,7 +340,7 @@ json_path_handler_base::gen_schema(yajlpp_gen_context& ygc) const
 
         if (this->jph_is_pattern_property) {
             ygc.ygc_path.emplace_back(fmt::format(
-                FMT_STRING("<{}>"), this->jph_regex->name_for_capture(0)));
+                FMT_STRING("<{}>"), this->jph_regex->get_name_for_capture(1)));
         } else {
             ygc.ygc_path.emplace_back(this->jph_property);
         }
@@ -365,7 +356,8 @@ json_path_handler_base::gen_schema(yajlpp_gen_context& ygc) const
         schema.gen("type");
 
         if (this->jph_is_array) {
-            if (this->jph_regex->p_pattern.find("#?") == std::string::npos) {
+            if (this->jph_regex->get_pattern().find("#?") == std::string::npos)
+            {
                 schema.gen("array");
             } else {
                 yajlpp_array type_array(ygc.ygc_handle);
@@ -386,7 +378,7 @@ json_path_handler_base::gen_schema(yajlpp_gen_context& ygc) const
             schema.gen("examples");
 
             yajlpp_array example_array(ygc.ygc_handle);
-            for (auto& ex : this->jph_examples) {
+            for (const auto& ex : this->jph_examples) {
                 example_array.gen(ex);
             }
         }
@@ -458,8 +450,8 @@ json_path_handler_base::gen_schema_type(yajlpp_gen_context& ygc) const
 
 void
 json_path_handler_base::walk(
-    const std::function<
-        void(const json_path_handler_base&, const std::string&, void*)>& cb,
+    const std::function<void(
+        const json_path_handler_base&, const std::string&, const void*)>& cb,
     void* root,
     const std::string& base) const
 {
@@ -468,11 +460,11 @@ json_path_handler_base::walk(
     if (this->jph_path_provider) {
         this->jph_path_provider(root, local_paths);
 
-        for (auto& lpath : local_paths) {
+        for (const auto& lpath : local_paths) {
             cb(*this,
                fmt::format(FMT_STRING("{}{}{}"),
                            base,
-                           lpath,
+                           json_ptr::encode_str(lpath),
                            this->jph_children ? "/" : ""),
                nullptr);
         }
@@ -480,6 +472,12 @@ json_path_handler_base::walk(
             local_paths.clear();
             this->jph_path_provider(root, local_paths);
         }
+        if (this->jph_field_getter) {
+            const auto* field = this->jph_field_getter(root, nonstd::nullopt);
+            if (field != nullptr) {
+                cb(*this, base, field);
+            }
+        }
     } else {
         local_paths.emplace_back(this->jph_property);
 
@@ -487,16 +485,17 @@ json_path_handler_base::walk(
         if (this->jph_children) {
             full_path += "/";
         }
+
         cb(*this, full_path, nullptr);
     }
 
     if (this->jph_children) {
         for (const auto& lpath : local_paths) {
             for (const auto& jph : this->jph_children->jpc_children) {
-                static const auto POSS_SRC
+                static const intern_string_t POSS_SRC
                     = intern_string::lookup("possibilities");
 
-                std::string full_path = base + lpath;
+                std::string full_path = base + json_ptr::encode_str(lpath);
                 if (this->jph_children) {
                     full_path += "/";
                 }
@@ -509,16 +508,25 @@ json_path_handler_base::walk(
 
                 ypc.set_path(full_path).with_obj(root).update_callbacks();
                 if (this->jph_obj_provider) {
-                    std::string full_path = lpath + "/";
-                    pcre_input pi(full_path);
+                    static thread_local auto md
+                        = lnav::pcre2pp::match_data::unitialized();
 
-                    if (!this->jph_regex->match(ypc.ypc_pcre_context, pi)) {
+                    const auto short_path = json_ptr::encode_str(lpath) + "/";
+
+                    if (!this->jph_regex->capture_from(short_path)
+                             .into(md)
+                             .matches()
+                             .ignore_error())
+                    {
+                        log_error(
+                            "path-handler regex (%s) does not match path: "
+                            "%s",
+                            this->jph_regex->get_pattern().c_str(),
+                            full_path.c_str());
                         ensure(false);
                     }
                     child_root = this->jph_obj_provider(
-                        {{ypc.ypc_pcre_context, pi},
-                         yajlpp_provider_context::nindex},
-                        root);
+                        {&md, yajlpp_provider_context::nindex}, root);
                 }
 
                 jph.walk(cb, child_root, full_path);
@@ -526,7 +534,7 @@ json_path_handler_base::walk(
         }
     } else {
         for (auto& lpath : local_paths) {
-            void* field = nullptr;
+            const void* field = nullptr;
 
             if (this->jph_field_getter) {
                 field = this->jph_field_getter(root, lpath);
@@ -540,7 +548,7 @@ nonstd::optional<int>
 json_path_handler_base::to_enum_value(const string_fragment& sf) const
 {
     for (int lpc = 0; this->jph_enum_values[lpc].first; lpc++) {
-        const enum_value_t& ev = this->jph_enum_values[lpc];
+        const auto& ev = this->jph_enum_values[lpc];
 
         if (sf == ev.first) {
             return ev.second;
@@ -683,8 +691,6 @@ yajlpp_parse_context::update_callbacks(const json_path_container* orig_handlers,
     }
 
     this->ypc_sibling_handlers = orig_handlers;
-    pcre_input pi(&this->ypc_path[0], 0, this->ypc_path.size() - 1);
-
     this->ypc_callbacks = DEFAULT_CALLBACKS;
 
     if (handlers == nullptr) {
@@ -709,12 +715,18 @@ yajlpp_parse_context::update_callbacks(const json_path_container* orig_handlers,
         }
     }
 
+    auto path_frag = string_fragment::from_byte_range(
+        this->ypc_path.data(), 1 + child_start, this->ypc_path.size() - 1);
     for (const auto& jph : handlers->jpc_children) {
-        pi.reset(&this->ypc_path[1 + child_start],
-                 0,
-                 this->ypc_path.size() - 2 - child_start);
-        if (jph.jph_regex->match(this->ypc_pcre_context, pi)) {
-            pcre_context::capture_t* cap = this->ypc_pcre_context.all();
+        static thread_local auto md = lnav::pcre2pp::match_data::unitialized();
+
+        if (jph.jph_regex->capture_from(path_frag)
+                .into(md)
+                .matches()
+                .ignore_error()
+            && (md.remaining().empty() || md.remaining().startswith("/")))
+        {
+            auto cap = md[0].value();
 
             if (jph.jph_is_array) {
                 this->ypc_array_handler_count += 1;
@@ -724,31 +736,24 @@ yajlpp_parse_context::update_callbacks(const json_path_container* orig_handlers,
                     ? static_cast<size_t>(-1)
                     : this->ypc_array_index[this->ypc_array_handler_count - 1];
 
-                if ((1 + child_start + cap->c_end
-                     != (int) this->ypc_path.size() - 1)
+                if ((cap.sf_end != (int) this->ypc_path.size() - 1)
                     && (!jph.is_array()
                         || index != yajlpp_provider_context::nindex))
                 {
                     this->ypc_obj_stack.push(jph.jph_obj_provider(
-                        {{this->ypc_pcre_context, pi}, index, this},
-                        this->ypc_obj_stack.top()));
+                        {&md, index, this}, this->ypc_obj_stack.top()));
                 }
             }
 
             if (jph.jph_children) {
                 this->ypc_handler_stack.emplace_back(&jph);
 
-                if (1 + child_start + cap->c_end
-                    != (int) this->ypc_path.size() - 1)
-                {
-                    this->update_callbacks(jph.jph_children,
-                                           1 + child_start + cap->c_end);
+                if (cap.sf_end != (int) this->ypc_path.size() - 1) {
+                    this->update_callbacks(jph.jph_children, cap.sf_end);
                     return;
                 }
             } else {
-                if (1 + child_start + cap->c_end
-                    != (int) this->ypc_path.size() - 1)
-                {
+                if (cap.sf_end != (int) this->ypc_path.size() - 1) {
                     continue;
                 }
 
@@ -766,6 +771,9 @@ yajlpp_parse_context::update_callbacks(const json_path_container* orig_handlers,
                 this->ypc_callbacks.yajl_integer
                     = jph.jph_callbacks.yajl_integer;
             }
+            if (jph.jph_callbacks.yajl_number != nullptr) {
+                this->ypc_callbacks.yajl_number = jph.jph_callbacks.yajl_number;
+            }
             if (jph.jph_callbacks.yajl_double != nullptr) {
                 this->ypc_callbacks.yajl_double = jph.jph_callbacks.yajl_double;
             }
@@ -953,13 +961,16 @@ yajlpp_parse_context::handle_unused_or_delete(void* ctx)
     if (!ypc->ypc_handler_stack.empty()
         && ypc->ypc_handler_stack.back()->jph_obj_deleter)
     {
-        pcre_context_static<30> pc;
+        static thread_local auto md = lnav::pcre2pp::match_data::unitialized();
+
         auto key_start = ypc->ypc_path_index_stack.back();
-        pcre_input pi(&ypc->ypc_path[key_start + 1],
-                      0,
-                      ypc->ypc_path.size() - key_start - 2);
-        yajlpp_provider_context provider_ctx{{pc, pi}, static_cast<size_t>(-1)};
-        ypc->ypc_handler_stack.back()->jph_regex->match(pc, pi);
+        auto path_frag = string_fragment::from_byte_range(
+            ypc->ypc_path.data(), key_start + 1, ypc->ypc_path.size() - 1);
+        yajlpp_provider_context provider_ctx{&md, static_cast<size_t>(-1)};
+        ypc->ypc_handler_stack.back()
+            ->jph_regex->capture_from(path_frag)
+            .into(md)
+            .matches();
 
         ypc->ypc_handler_stack.back()->jph_obj_deleter(
             provider_ctx, ypc->ypc_obj_stack.top());
@@ -1040,6 +1051,7 @@ yajlpp_parse_context::parse_doc(const string_fragment& sf)
 
     auto rc = yajl_parse(this->ypc_handle, this->ypc_json_text, sf.length());
     size_t consumed = yajl_get_bytes_consumed(this->ypc_handle);
+    this->ypc_total_consumed += consumed;
     this->ypc_line_number += std::count(
         &this->ypc_json_text[0], &this->ypc_json_text[consumed], '\n');
 
@@ -1106,7 +1118,7 @@ yajlpp_parse_context::reset(const struct json_path_container* handlers)
 }
 
 void
-yajlpp_parse_context::set_static_handler(json_path_handler_base& jph)
+yajlpp_parse_context::set_static_handler(const json_path_handler_base& jph)
 {
     this->ypc_path.clear();
     this->ypc_path.push_back('/');
@@ -1123,6 +1135,11 @@ yajlpp_parse_context::set_static_handler(json_path_handler_base& jph)
     if (jph.jph_callbacks.yajl_integer != nullptr) {
         this->ypc_callbacks.yajl_integer = jph.jph_callbacks.yajl_integer;
     }
+    if (jph.jph_callbacks.yajl_number != nullptr) {
+        this->ypc_callbacks.yajl_number = jph.jph_callbacks.yajl_number;
+    } else {
+        this->ypc_callbacks.yajl_number = nullptr;
+    }
     if (jph.jph_callbacks.yajl_double != nullptr) {
         this->ypc_callbacks.yajl_double = jph.jph_callbacks.yajl_double;
     }
@@ -1309,6 +1326,40 @@ yajlpp_parse_context::get_snippet() const
         .with_line(line_number);
 }
 
+void
+json_path_handler_base::validate_string(yajlpp_parse_context& ypc,
+                                        string_fragment sf) const
+{
+    if (this->jph_pattern) {
+        if (!this->jph_pattern->find_in(sf).ignore_error()) {
+            this->report_pattern_error(&ypc, sf.to_string());
+        }
+    }
+    if (sf.empty() && this->jph_min_length > 0) {
+        ypc.report_error(lnav::console::user_message::error(
+                             attr_line_t("invalid value for option ")
+                                 .append_quoted(lnav::roles::symbol(
+                                     ypc.get_full_path().to_string())))
+                             .with_reason("empty values are not allowed")
+                             .with_snippet(ypc.get_snippet())
+                             .with_help(this->get_help_text(&ypc)));
+    } else if (sf.length() < this->jph_min_length) {
+        ypc.report_error(
+            lnav::console::user_message::error(
+                attr_line_t()
+                    .append_quoted(sf)
+                    .append(" is not a valid value for option ")
+                    .append_quoted(
+                        lnav::roles::symbol(ypc.get_full_path().to_string())))
+                .with_reason(attr_line_t("value must be at least ")
+                                 .append(lnav::roles::number(
+                                     fmt::to_string(this->jph_min_length)))
+                                 .append(" characters long"))
+                .with_snippet(ypc.get_snippet())
+                .with_help(this->get_help_text(&ypc)));
+    }
+}
+
 void
 json_path_handler_base::report_pattern_error(yajlpp_parse_context* ypc,
                                              const std::string& value_str) const
@@ -1326,6 +1377,33 @@ json_path_handler_base::report_pattern_error(yajlpp_parse_context* ypc,
             .with_help(this->get_help_text(ypc)));
 }
 
+void
+json_path_handler_base::report_tz_error(yajlpp_parse_context* ypc,
+                                        const std::string& value_str,
+                                        const char* msg) const
+{
+    auto help_al = attr_line_t()
+                       .append(lnav::roles::h2("Available time zones"))
+                       .append("\n");
+
+    try {
+        for (const auto& tz : date::get_tzdb().zones) {
+            help_al.append("    ")
+                .append(lnav::roles::symbol(tz.name()))
+                .append("\n");
+        }
+    } catch (const std::runtime_error& e) {
+        log_error("unable to load timezones: %s", e.what());
+    }
+
+    ypc->report_error(lnav::console::user_message::error(
+                          attr_line_t().append_quoted(value_str).append(
+                              " is not a valid timezone"))
+                          .with_snippet(ypc->get_snippet())
+                          .with_reason(msg)
+                          .with_help(help_al));
+}
+
 attr_line_t
 json_path_handler_base::get_help_text(const std::string& full_path) const
 {
@@ -1377,8 +1455,6 @@ void
 json_path_handler_base::report_min_value_error(yajlpp_parse_context* ypc,
                                                long long value) const
 {
-    const auto* jph = ypc->ypc_current_handler;
-
     ypc->report_error(
         lnav::console::user_message::error(
             attr_line_t()
@@ -1388,9 +1464,9 @@ json_path_handler_base::report_min_value_error(yajlpp_parse_context* ypc,
                     lnav::roles::symbol(ypc->get_full_path().to_string())))
             .with_reason(attr_line_t("value must be greater than or equal to ")
                              .append(lnav::roles::number(
-                                 fmt::to_string(jph->jph_min_value))))
+                                 fmt::to_string(this->jph_min_value))))
             .with_snippet(ypc->get_snippet())
-            .with_help(jph->get_help_text(ypc)));
+            .with_help(this->get_help_text(ypc)));
 }
 
 void
@@ -1426,35 +1502,11 @@ json_path_handler_base::report_enum_error(yajlpp_parse_context* ypc,
 }
 
 void
-json_path_handler_base::report_regex_value_error(
-    yajlpp_parse_context* ypc,
-    const std::string& value,
-    const pcrepp::compile_error& pcre_error) const
+json_path_handler_base::report_error(yajlpp_parse_context* ypc,
+                                     const std::string& value,
+                                     lnav::console::user_message um) const
 {
-    attr_line_t pcre_error_content{value};
-
-    lnav::snippets::regex_highlighter(pcre_error_content,
-                                      pcre_error_content.length(),
-                                      line_range{
-                                          0,
-                                          (int) pcre_error_content.length(),
-                                      });
-    pcre_error_content.append("\n")
-        .append(pcre_error.ce_offset, ' ')
-        .append(lnav::roles::error("^ "))
-        .append(lnav::roles::error(pcre_error.ce_msg))
-        .with_attr_for_all(VC_ROLE.value(role_t::VCR_QUOTED_CODE));
-    ypc->report_error(lnav::console::user_message::error(
-                          attr_line_t()
-                              .append_quoted(value)
-                              .append(" is not a valid regular expression for "
-                                      "property ")
-                              .append_quoted(lnav::roles::symbol(
-                                  ypc->get_full_path().to_string())))
-                          .with_reason(pcre_error.ce_msg)
-                          .with_snippet(ypc->get_snippet())
-                          .with_snippet(lnav::console::snippet::from(
-                              ypc->get_full_path(), pcre_error_content))
+    ypc->report_error(um.with_snippet(ypc->get_snippet())
                           .with_help(this->get_help_text(ypc)));
 }
 
@@ -1488,7 +1540,7 @@ json_path_container::gen_properties(yajlpp_gen_context& ygc) const
         {
             yajlpp_map properties(ygc.ygc_handle);
 
-            for (auto& child_handler : this->jpc_children) {
+            for (const auto& child_handler : this->jpc_children) {
                 if (child_handler.jph_is_pattern_property) {
                     continue;
                 }
@@ -1552,5 +1604,19 @@ yajlpp_gen::to_string_fragment()
 
     yajl_gen_get_buf(this->yg_handle.in(), &buf, &len);
 
-    return string_fragment((const char*) buf, 0, len);
+    return string_fragment::from_bytes(buf, len);
 }
+
+namespace yajlpp {
+
+auto_mem<yajl_handle_t>
+alloc_handle(const yajl_callbacks* cb, void* cu)
+{
+    auto_mem<yajl_handle_t> retval(yajl_free);
+
+    retval = yajl_alloc(cb, nullptr, cu);
+
+    return retval;
+}
+
+}  // namespace yajlpp
\ No newline at end of file
